from flask import request
from flask_restx import Namespace, fields, Resource

from lib.CVEs import CveHandler
from lib.Config import Configuration
from lib.Query import qcvesForCPE
from web.restapi.cpe_convert import message

config = Configuration()

api = Namespace("cve", description="Endpoints for requesting cve information", path="/")

access_entry = api.model(
    "access",
    {
        "authentication": fields.String(
            description="This metric describes the level of privileges an attacker must possess before successfully "
            "exploiting the vulnerability.",
            example="SINGLE",
        ),
        "complexity": fields.String(
            description="The Attack Complexity metric describes the conditions beyond the attacker's control that must "
            "exist in order to exploit the vulnerability. As described below, such conditions may require "
            "the collection of more information about the target, the presence of certain system "
            "configuration settings, or computational exceptions.",
            example="LOW",
        ),
        "vector": fields.String(
            description="This metric reflects the context by which vulnerability exploitation is possible. This metric "
            "value (and consequently the Base score) will be larger the more remote (logically, "
            "and physically) an attacker can be in order to exploit the vulnerable component.",
            example="NETWORK",
        ),
    },
)

access_V3_entry = api.model(
    "access_V3",
    {
        "attackvector": fields.String(
            description="This metric reflects the context by which vulnerability exploitation is possible. This metric "
            "value (and consequently the Base Score) will be larger the more remote (logically, "
            "and physically) an attacker can be in order to exploit the vulnerable component. The "
            "assumption is that the number of potential attackers for a vulnerability that could be "
            "exploited from across a network is larger than the number of potential attackers that could "
            "exploit a vulnerability requiring physical access to a device, and therefore warrants a "
            "greater Base Score.",
            example="NETWORK",
        ),
        "attackcomplexity": fields.String(
            description="This metric describes the conditions beyond the attacker’s control that must exist in order "
            "to exploit the vulnerability. As described below, such conditions may require the collection "
            "of more information about the target, or computational exceptions. Importantly, the "
            "assessment of this metric excludes any requirements for user interaction in order to exploit "
            "the vulnerability (such conditions are captured in the User Interaction metric). If a "
            "specific configuration is required for an attack to succeed, the Base metrics should be "
            "scored assuming the vulnerable component is in that configuration. The Base Score is greatest "
            "for the least complex attacks.",
            example="LOW",
        ),
        "privilegesrequired": fields.String(
            description="This metric describes the level of privileges an attacker must possess before successfully "
            "exploiting the vulnerability. The Base Score is greatest if no privileges are required.",
            example="NONE",
        ),
        "userinteraction": fields.String(
            description="This metric captures the requirement for a human user, other than the attacker, to "
            "participate in the successful compromise of the vulnerable component. This metric determines whether the "
            "vulnerability can be exploited solely at the will of the attacker, or whether a separate user "
            "(or user-initiated process) must participate in some manner. The Base Score is greatest when "
            "no user interaction is required.",
            example="REQUIRED",
        ),
        "scope": fields.String(
            description="The Scope metric captures whether a vulnerability in one vulnerable component impacts "
            "resources in components beyond its security scope. Formally, a security authority is a "
            "mechanism (e.g., an application, an operating system, firmware, a sandbox environment) that "
            "defines and enforces access control in terms of how certain subjects/actors (e.g., human "
            "users, processes) can access certain restricted objects/resources (e.g., files, CPU, memory) "
            "in a controlled manner. All the subjects and objects under the jurisdiction of a single "
            "security authority are considered to be under one security scope. If a vulnerability in a "
            "vulnerable component can affect a component which is in a different security scope than the "
            "vulnerable component, a Scope change occurs. Intuitively, whenever the impact of a "
            "vulnerability breaches a security/trust boundary and impacts components outside the security "
            "scope in which vulnerable component resides, a Scope change occurs. The security scope of a "
            "component encompasses other components that provide functionality solely to that component, "
            "even if these other components have their own security authority. For example, a database "
            "used solely by one application is considered part of that application’s security scope even "
            "if the database has its own security authority, e.g., a mechanism controlling access to "
            "database records based on database users and associated database privileges. The Base Score "
            "is greatest when a scope change occurs.",
            example="CHANGED",
        ),
    },
)

access_V4_entry = api.model(
    "access_V4",
    {
        "attackvector": fields.String(
            description="This metric reflects the context by which vulnerability exploitation is possible. This metric "
            "value (and consequently the Base Score) will be larger the more remote (logically, "
            "and physically) an attacker can be in order to exploit the vulnerable component. The "
            "assumption is that the number of potential attackers for a vulnerability that could be "
            "exploited from across a network is larger than the number of potential attackers that could "
            "exploit a vulnerability requiring physical access to a device, and therefore warrants a "
            "greater Base Score.",
            example="NETWORK",
        ),
        "attackcomplexity": fields.String(
            description="This metric describes the conditions beyond the attacker’s control that must exist in order "
            "to exploit the vulnerability. As described below, such conditions may require the collection "
            "of more information about the target, or computational exceptions. Importantly, the "
            "assessment of this metric excludes any requirements for user interaction in order to exploit "
            "the vulnerability (such conditions are captured in the User Interaction metric). If a "
            "specific configuration is required for an attack to succeed, the Base metrics should be "
            "scored assuming the vulnerable component is in that configuration. The Base Score is greatest "
            "for the least complex attacks.",
            example="LOW",
        ),
        "privilegesrequired": fields.String(
            description="This metric describes the level of privileges an attacker must possess before successfully "
            "exploiting the vulnerability. The Base Score is greatest if no privileges are required.",
            example="NONE",
        ),
        "userinteraction": fields.String(
            description="This metric captures the requirement for a human user, other than the attacker, to "
            "participate in the successful compromise of the vulnerable component. This metric determines whether the "
            "vulnerability can be exploited solely at the will of the attacker, or whether a separate user "
            "(or user-initiated process) must participate in some manner. The Base Score is greatest when "
            "no user interaction is required.",
            example="REQUIRED",
        ),
    },
)

impact_entry = api.model(
    "Impact",
    {
        "availability": fields.String(
            description="This metric measures the impact to the availability of the impacted component resulting "
            "from a successfully exploited vulnerability. While the Confidentiality and Integrity impact "
            "metrics apply to the loss of confidentiality or integrity of data (e.g., information, files) "
            "used by the impacted component, this metric refers to the loss of availability of the "
            "impacted component itself, such as a networked service (e.g., web, database, email). Since "
            "availability refers to the accessibility of information resources, attacks that consume "
            "network bandwidth, processor cycles, or disk space all impact the availability of an impacted "
            "component.",
            example="COMPLETE",
        ),
        "confidentiality": fields.String(
            description="This metric measures the impact to the confidentiality of the information resources managed "
            "by a software component due to a successfully exploited vulnerability. Confidentiality refers "
            "to limiting information access and disclosure to only authorized users, as well as preventing "
            "access by, or disclosure to, unauthorized ones.",
            example="COMPLETE",
        ),
        "integrity": fields.String(
            description="This metric measures the impact to integrity of a successfully exploited vulnerability. "
            "Integrity refers to the trustworthiness and veracity of information.",
            example="COMPLETE",
        ),
    },
)

impact_V3_entry = api.model(
    "Impact",
    {
        "availability": fields.String(
            description="This metric measures the impact to the availability of the impacted component resulting "
            "from a successfully exploited vulnerability. While the Confidentiality and Integrity impact "
            "metrics apply to the loss of confidentiality or integrity of data (e.g., information, files) "
            "used by the impacted component, this metric refers to the loss of availability of the "
            "impacted component itself, such as a networked service (e.g., web, database, email). Since "
            "availability refers to the accessibility of information resources, attacks that consume "
            "network bandwidth, processor cycles, or disk space all impact the availability of an impacted "
            "component.",
            example="HIGH",
        ),
        "confidentiality": fields.String(
            description="This metric measures the impact to the confidentiality of the information resources managed "
            "by a software component due to a successfully exploited vulnerability. Confidentiality refers "
            "to limiting information access and disclosure to only authorized users, as well as preventing "
            "access by, or disclosure to, unauthorized ones.",
            example="HIGH",
        ),
        "integrity": fields.String(
            description="This metric measures the impact to integrity of a successfully exploited vulnerability. "
            "Integrity refers to the trustworthiness and veracity of information.",
            example="HIGH",
        ),
    },
)


impact_V4_entry = api.model(
    "Impact",
    {
        "vulnerable_system_availability": fields.String(
            description="This metric measures the impact to the availability of the VULNERABLE SYSTEM resulting from "
            "a successfully exploited vulnerability. While the Confidentiality and Integrity impact "
            "metrics apply to the loss of confidentiality or integrity of data (e.g., information, "
            "files) used by the system, this metric refers to the loss of availability of the impacted "
            "system itself, such as a networked service (e.g., web, database, email). Since availability "
            "refers to the accessibility of information resources, attacks that consume network "
            "bandwidth, processor cycles, or disk space all impact the availability of a system.",
            example="HIGH",
        ),
        "vulnerable_system_confidentiality": fields.String(
            description="This metric measures the impact to the confidentiality of the information managed by the "
            "VULNERABLE SYSTEM due to a successfully exploited vulnerability. Confidentiality refers to "
            "limiting information access and disclosure to only authorized users, as well as preventing "
            "access by, or disclosure to, unauthorized ones.",
            example="HIGH",
        ),
        "vulnerable_system_integrity": fields.String(
            description="This metric measures the impact to integrity of a successfully exploited vulnerability. "
            "Integrity refers to the trustworthiness and veracity of information. Integrity of the "
            "VULNERABLE SYSTEM is impacted when an attacker makes unauthorized modification of system "
            "data. Integrity is also impacted when a system user can repudiate critical actions taken in "
            "the context of the system (e.g. due to insufficient logging).",
            example="HIGH",
        ),
        "subsequent_system_confidentiality": fields.String(
            description="This metric measures the impact to the confidentiality of the information managed by the "
            "SUBSEQUENT SYSTEM due to a successfully exploited vulnerability. Confidentiality refers to "
            "limiting information access and disclosure to only authorized users, as well as preventing "
            "access by, or disclosure to, unauthorized ones.",
            example="HIGH",
        ),
        "subsequent_system_integrity": fields.String(
            description="This metric measures the impact to integrity of a successfully exploited vulnerability. "
            "Integrity refers to the trustworthiness and veracity of information. Integrity of the "
            "SUBSEQUENT SYSTEM is impacted when an attacker makes unauthorized modification of system "
            "data. Integrity is also impacted when a system user can repudiate critical actions taken in "
            "the context of the system (e.g. due to insufficient logging).",
            example="HIGH",
        ),
        "subsequent_system_availability": fields.String(
            description="This metric measures the impact to the availability of the SUBSEQUENT SYSTEM resulting from "
            "a successfully exploited vulnerability. While the Confidentiality and Integrity impact "
            "metrics apply to the loss of confidentiality or integrity of data (e.g., information, "
            "files) used by the system, this metric refers to the loss of availability of the impacted "
            "system itself, such as a networked service (e.g., web, database, email). Since availability "
            "refers to the accessibility of information resources, attacks that consume network "
            "bandwidth, processor cycles, or disk space all impact the availability of a system.",
            example="HIGH",
        ),
    },
)

tax_inner = api.model(
    "TaxonomyInner",
    {
        "Entry_ID": fields.String(description="Taxonomy entry id", example="1499.001"),
        "Entry_Name": fields.String(
            description="Taxonomy name",
            example="Endpoint Denial of Service:OS Exhaustion Flood",
        ),
        "URL": fields.String(
            description="Taxonomy URL",
            example="https://attack.mitre.org/techniques/T1499/001",
        ),
    },
)

tax_wild = fields.Wildcard(
    fields.Nested(tax_inner), description="Taxonomy mapping id", skip_none=True
)

taxonomy_entry = api.model("documents", {"*": tax_wild})

taxonomy = api.model(
    "Taxonomy",
    {
        "ATTACK": fields.Nested(
            taxonomy_entry,
            skip_none=True,
            description="Mitre Att&ck taxonomy",
            example={
                "1499_001": {
                    "Entry_ID": "1499.001",
                    "Entry_Name": "Endpoint Denial of Service:OS Exhaustion Flood",
                    "URL": "https://attack.mitre.org/techniques/T1499/001",
                },
                "1499_004": {
                    "Entry_ID": "1499.004",
                    "Entry_Name": "Endpoint Denial of Service:Application or System Exploitation",
                    "URL": "https://attack.mitre.org/techniques/T1499/004",
                },
            },
        )
    },
)

execution_inner = api.model(
    "ExecutionInner",
    {
        "Phase": fields.String(description="Execution phase", example="Explore"),
        "Description": fields.String(
            description="Execution phase description",
            example="[Determine application's/system's password policy] Determine the password policies of the target "
            "application/system.",
        ),
        "Techniques": fields.List(
            fields.String,
            description="A list of used techniques during the phase",
            example=[
                "Determine minimum and maximum allowed password lengths.",
                "Determine format of allowed passwords (whether they are required or allowed to contain numbers, "
                "special characters, etc.).",
                "Determine account lockout policy (a strict account lockout policy will prevent brute force attacks).",
            ],
        ),
    },
)

execution_wild = fields.Wildcard(
    fields.Nested(execution_inner),
    description="Execution phase mapping id",
    skip_none=True,
)

execution_entry = api.model("documents", {"*": execution_wild})

capec_entry = api.model(
    "Capec_entry",
    {
        "id": fields.String(description="ID number of the CAPEC", example="242"),
        "name": fields.String(
            description="Name of the CAPEC", example="Code Injection"
        ),
        "prerequisites": fields.String(
            description="Prerequisites of the CAPEC",
            example="The target software does not validate user-controlled input such that the execution of a process "
            "may be altered by sending code in through legitimate data channels, using no other mechanism.",
        ),
        "related_weakness": fields.List(
            fields.String,
            description="List with related CWE numbers",
            example=["94", "270", "271"],
        ),
        "related_capecs": fields.List(
            fields.String,
            description="List with related CAPEC numbers",
            example=["112", "151", "560", "600", "653"],
        ),
        "solutions": fields.String(
            description="Solutions for mitigating the CAPEC",
            example="Utilize strict type, character, and encoding enforcement Ensure all input content that is "
            "delivered to client is sanitized against an acceptable content specification. Perform input "
            "validation for all content. Enforce regular patching of software.",
        ),
        "summary": fields.String(
            description="Summary of the CAPEC",
            example="An adversary exploits a weakness in input validation on the target to inject new code into that "
            "which is currently executing. This differs from code inclusion in that code inclusion involves "
            "the addition or replacement of a reference to a code file, which is subsequently loaded by the "
            "target and used as part of the code of some application.",
        ),
        "loa": fields.String(description="Likelyhood of attack", example="Medium"),
        "typical_severity": fields.String(
            description="Typical severity", example="High"
        ),
        "taxonomy": fields.Nested(
            taxonomy,
            skip_none=True,
            description="Taxonomy linked to this CAPEC",
            example={
                "ATTACK": {
                    "1499_001": {
                        "Entry_ID": "1499.001",
                        "Entry_Name": "Endpoint Denial of Service:OS Exhaustion Flood",
                        "URL": "https://attack.mitre.org/techniques/T1499/001",
                    },
                    "1499_004": {
                        "Entry_ID": "1499.004",
                        "Entry_Name": "Endpoint Denial of Service:Application or System Exploitation",
                        "URL": "https://attack.mitre.org/techniques/T1499/004",
                    },
                }
            },
        ),
        "execution_flow": fields.Nested(
            execution_entry,
            skip_none=True,
            description="Phased description of attack execution flow",
            example={
                "1": {
                    "Phase": "Explore",
                    "Description": "[Determine application's/system's password policy] Determine the password policies "
                    "of the target application/system.",
                    "Techniques": [
                        "Determine minimum and maximum allowed password lengths.",
                        "Determine format of allowed passwords (whether they are required or allowed to contain "
                        "numbers, special characters, etc.).",
                        "Determine account lockout policy (a strict account lockout policy will prevent brute "
                        "force attacks).",
                    ],
                },
                "2": {
                    "Phase": "Exploit",
                    "Description": "[Brute force password] Given the finite space of possible passwords dictated by "
                    "the password policy determined in the previous step, try all possible passwords "
                    "for a known user ID until application/system grants access.",
                    "Techniques": [
                        "Manually or automatically enter all possible passwords through the application/system's "
                        "interface. In most systems, start with the shortest and simplest possible passwords, because "
                        "most users tend to select such passwords if allowed to do so.",
                        "Perform an offline dictionary attack or a rainbow table attack against a known password hash.",
                    ],
                },
            },
        ),
    },
)

refmap_inner = fields.List(
    fields.String,
    description="Reference mappings",
    example=["https://pastebin.com/QTev1TjM"],
)

refmap_wild = fields.Wildcard(refmap_inner, description="Reference mapping id")

refmap = api.model("documents", {"*": refmap_wild})

vuln_conf = api.model(
    "VulnConf",
    {
        "id": fields.String(
            description="ID of the vulnerable configuration",
            example="cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
        ),
        "title": fields.String(
            description="Title of the vulnerable configuration",
            example="cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
        ),
    },
)

contributors = api.model(
    "Contributors",
    {
        "name": fields.String(description="Contributor name", example="Dragos Prisaca"),
        "organization": fields.String(
            description="Organization name", example="Symantec Corporation"
        ),
    },
)

def_extensions = api.model(
    "DefExtensions",
    {
        "comment": fields.String(
            description="Comment",
            example="Microsoft Windows Vista (32-bit) Service Pack 1 is installed",
        ),
        "oval": fields.String(
            description="Oval reference", example="oval:org.mitre.oval:def:4873"
        ),
    },
)

oval_entry = api.model(
    "OvalEntry",
    {
        "accepted": fields.DateTime(
            description="Oval Date Time", example="2014-06-16T04:00:06.077-04:00"
        ),
        "class": fields.String(description="Oval class", example="vulnerability"),
        "contributors": fields.List(
            fields.Nested(contributors),
            skip_none=True,
            description="List with contributors",
            example=[
                {"name": "Dragos Prisaca", "organization": "Symantec Corporation"},
                {"name": "Chandan S", "organization": "SecPod Technologies"},
                {"name": "Maria Kedovskaya", "organization": "ALTX-SOFT"},
                {"name": "Maria Mikhno", "organization": "ALTX-SOFT"},
            ],
        ),
        "definition_extensions": fields.List(
            fields.Nested(def_extensions),
            skip_none=True,
            description="List with Definition extensions",
            example=[
                {
                    "comment": "Microsoft Windows Vista (32-bit) Service Pack 1 is installed",
                    "oval": "oval:org.mitre.oval:def:4873",
                },
                {
                    "comment": "Microsoft Windows Vista x64 Edition Service Pack 1 is installed",
                    "oval": "oval:org.mitre.oval:def:5254",
                },
                {
                    "comment": "Microsoft Windows Vista (32-bit) Service Pack 2 is installed",
                    "oval": "oval:org.mitre.oval:def:6124",
                },
                {
                    "comment": "Microsoft Windows Vista x64 Edition Service Pack 2 is installed",
                    "oval": "oval:org.mitre.oval:def:5594",
                },
                {
                    "comment": "Microsoft Windows 7 (32-bit) is installed",
                    "oval": "oval:org.mitre.oval:def:6165",
                },
                {
                    "comment": "Microsoft Windows 7 x64 Edition is installed",
                    "oval": "oval:org.mitre.oval:def:5950",
                },
                {
                    "comment": "Microsoft Windows 7 (32-bit) Service Pack 1 is installed",
                    "oval": "oval:org.mitre.oval:def:12292",
                },
                {
                    "comment": "Microsoft Windows 7 x64 Service Pack 1 is installed",
                    "oval": "oval:org.mitre.oval:def:12627",
                },
            ],
        ),
        "description": fields.String(
            description="Oval description",
            example="The Bluetooth Stack 2.1 in Microsoft Windows Vista SP1 and SP2 and Windows 7 Gold and SP1 does "
            "not prevent access to objects in memory that (1) were not properly initialized or (2) have "
            "been deleted, which allows remote attackers to execute arbitrary code via crafted Bluetooth "
            "packets, aka 'Bluetooth Stack Vulnerability'.",
        ),
        "family": fields.String(description="Oval family", example="windows"),
        "id": fields.String(
            description="Oval id", example="oval:org.mitre.oval:def:12094"
        ),
        "status": fields.String(description="Oval status", example="accepted"),
        "submitted": fields.DateTime(
            description="Oval submitted", example="2011-07-12T13:00:00"
        ),
        "title": fields.String(
            description="Oval title", example="Bluetooth Stack Vulnerability"
        ),
        "version": fields.String(description="Oval version", example="52"),
    },
)

MsBulletin = api.model(
    "MsBulletin",
    {
        "bulletin_id": fields.String(
            description="MSBulletin id number", example="MS11-053"
        ),
        "bulletin_url": fields.String(
            description="MSBulletin url",
            example="https://www.windows.com/some_url_to_ms_bulletin",
        ),
        "date": fields.DateTime(
            description="MSBulletin date", example="2011-07-12T00:00:00"
        ),
        "impact": fields.String(
            description="MSBulletin impact", example="Remote Code Execution"
        ),
        "knowledgebase_id": fields.String(
            description="MSBulletin knowledgebase_id", example="2566220"
        ),
        "knowledgebase_url": fields.String(
            description="MSBulletin knowledgebase_url",
            example="https://www.windows.com/some_url_to_ms_bulletin_knowledge_base",
        ),
        "severity": fields.String(
            description="MSBulletin severity level", example="Critical"
        ),
        "title": fields.String(
            description="MSBulletin title",
            example="Vulnerability in Bluetooth Stack Could Allow Remote Code Execution",
        ),
    },
)

statements = api.model(
    "Statements",
    {
        "contributor": fields.String(
            description="Statement contributor", example="Tomas Hoger"
        ),
        "lastmodified": fields.Date(description="Statement date", example="2009-09-02"),
        "organization": fields.String(
            description="Statement organization", example="Red Hat"
        ),
        "statement": fields.String(
            description="Statement",
            example="This issue did not affect the versions of Linux kernel as shipped with Red Hat Enterprise Linux "
            "2.1, 3, 4, and Red Hat Enterprise MRG. This issue was addressed in Red Hat Enterprise Linux 5 "
            "by https://rhn.redhat.com/errata/RHSA-2009-1243.html",
        ),
    },
)

redhat_inner = api.model(
    "RHInner",
    {
        "id": fields.String(
            description="Red Hat Security Advisory id", example="RHSA-2009:1243"
        )
    },
)

redhat_wild = fields.Wildcard(
    fields.Nested(redhat_inner), description="Red Hat mapping id", skip_none=True
)

redhat = api.model("documents", {"*": redhat_wild})

redhat_entry = api.model(
    "RedHatEntry",
    {
        "rpms": fields.List(
            fields.String,
            description="List with affected packages",
            example=[
                "kernel-0:2.6.18-164.el5",
                "kernel-PAE-0:2.6.18-164.el5",
                "kernel-PAE-debuginfo-0:2.6.18-164.el5",
                "kernel-PAE-devel-0:2.6.18-164.el5",
                "kernel-debug-0:2.6.18-164.el5",
                "kernel-debug-debuginfo-0:2.6.18-164.el5",
                "kernel-debug-devel-0:2.6.18-164.el5",
                "kernel-debuginfo-0:2.6.18-164.el5",
                "kernel-debuginfo-common-0:2.6.18-164.el5",
                "kernel-devel-0:2.6.18-164.el5",
                "kernel-doc-0:2.6.18-164.el5",
                "kernel-headers-0:2.6.18-164.el5",
                "kernel-kdump-0:2.6.18-164.el5",
                "kernel-kdump-debuginfo-0:2.6.18-164.el5",
                "kernel-kdump-devel-0:2.6.18-164.el5",
                "kernel-xen-0:2.6.18-164.el5",
                "kernel-xen-debuginfo-0:2.6.18-164.el5",
                "kernel-xen-devel-0:2.6.18-164.el5",
            ],
        ),
        "advisories": fields.List(
            fields.Nested(redhat),
            skip_none=True,
            description="List with advisories",
            example=[{"rhsa": {"id": "RHSA-2009:1243"}}],
        ),
    },
)

epss_metric = api.model(
    "EPSSMetric",
    {
        "percentile": fields.Float(
            description="Relative EPSS ranking compared to all CVEs (0–1 scale)",
            example=0.06475,
        ),
        "lastModified": fields.DateTime(
            description="Timestamp when the EPSS metric was last updated",
            example="2025-12-27T13:37:00Z",
        ),
    },
)

cve_entry_base = api.model(
    "CVE_Entry_Base",
    {
        "modified": fields.DateTime(
            description="Modified date time", example="2020-10-07T15:01:00"
        ),
        "published": fields.DateTime(
            description="Published date time", example="2020-09-24T15:15:00"
        ),
        "access": fields.Nested(
            access_entry,
            skip_none=True,
            description="Access vectors",
            example={
                "authentication": "SINGLE",
                "complexity": "LOW",
                "vector": "NETWORK",
            },
        ),
        "assigner": fields.String(
            description="Assigner of the CVE", example="cve@mitre.org"
        ),
        "capec": fields.List(
            fields.Nested(capec_entry),
            skip_none=True,
            description="CAPEC's related to the CVE",
            example=[
                {
                    "id": "49",
                    "name": "Password Brute Forcing",
                    "prerequisites": "An adversary needs to know a username to target. The system uses password based "
                    "authentication as the one factor authentication mechanism. An application does "
                    "not have a password throttling mechanism in place. A good password throttling "
                    "mechanism will make it almost impossible computationally to brute force a "
                    "password as it may either lock out the user after a certain number of incorrect "
                    "attempts or introduce time out periods. Both of these would make a brute force "
                    "attack impractical.",
                    "related_weakness": [
                        "257",
                        "262",
                        "263",
                        "307",
                        "308",
                        "309",
                        "521",
                        "654",
                    ],
                    "related_capecs": ["112", "151", "560", "600", "653"],
                    "solutions": "Implement a password throttling mechanism. This mechanism should take into account "
                    "both the IP address and the log in name of the user. Put together a strong password "
                    "policy and make sure that all user created passwords comply with it. Alternatively "
                    "automatically generate strong passwords for users. Passwords need to be recycled to "
                    "prevent aging, that is every once in a while a new password must be chosen.",
                    "summary": "In this attack, the adversary tries every possible value for a password until they "
                    "succeed. A brute force attack, if feasible computationally, will always be successful "
                    "because it will essentially go through all possible passwords given the alphabet "
                    "used (lower case letters, upper case letters, numbers, symbols, etc.) and the maximum "
                    "length of the password. A system will be particularly vulnerable to this type of an "
                    "attack if it does not have a proper enforcement mechanism in place to ensure that "
                    "passwords selected by users are strong passwords that comply with an adequate password "
                    "policy. In practice a pure brute force attack on passwords is rarely used, unless the "
                    "password is suspected to be weak. Other password cracking methods exist that are far "
                    "more effective (e.g. dictionary attacks, rainbow tables, etc.). Knowing the password "
                    "policy on the system can make a brute force attack more efficient. For instance, if "
                    "the policy states that all passwords must be of a certain level, there is no need to "
                    "check smaller candidates.",
                    "loa": "Medium",
                    "typical_severity": "High",
                    "taxonomy": {
                        "ATTACK": {
                            "1110_001": {
                                "Entry_ID": "1110.001",
                                "Entry_Name": "Brute Force:Password Guessing",
                                "URL": "https://attack.mitre.org/techniques/T1110/001",
                            }
                        }
                    },
                    "execution_flow": {
                        "2": {
                            "Phase": "Exploit",
                            "Description": "[Brute force password] Given the finite space of possible passwords "
                            "dictated by the password policy determined in the previous step, try all "
                            "possible passwords for a known user ID until application/system grants "
                            "access.",
                            "Techniques": [
                                "Manually or automatically enter all possible passwords through the "
                                "application/system's interface. In most systems, start with the shortest and simplest "
                                "possible passwords, because most users tend to select such passwords if allowed to "
                                "do so.",
                                "Perform an offline dictionary attack or a rainbow table attack against a known "
                                "password hash.",
                            ],
                        },
                        "1": {
                            "Phase": "Explore",
                            "Description": "[Determine application's/system's password policy] Determine the password "
                            "policies of the target application/system.",
                            "Techniques": [
                                "Determine minimum and maximum allowed password lengths.",
                                "Determine format of allowed passwords (whether they are required or allowed to "
                                "contain numbers, special characters, etc.).",
                                "Determine account lockout policy (a strict account lockout policy will prevent brute "
                                "force attacks).",
                            ],
                        },
                    },
                },
                {
                    "id": "35",
                    "name": "Leverage Executable Code in Non-Executable Files",
                    "prerequisites": "The attacker must have the ability to modify non-executable files consumed by "
                    "the target software.",
                    "related_weakness": [
                        "264",
                        "270",
                        "272",
                        "275",
                        "282",
                        "59",
                        "714",
                        "94",
                        "95",
                        "96",
                        "97",
                    ],
                    "related_capecs": ["112", "151", "560", "600", "653"],
                    "solutions": "Design: Enforce principle of least privilege Design: Run server interfaces with a "
                    "non-root account and/or utilize chroot jails or other configuration techniques to "
                    "constrain privileges even if attacker gains some limited access to commands. "
                    "Implementation: Perform testing such as pen-testing and vulnerability scanning to "
                    "identify directories, programs, and interfaces that grant direct access to "
                    "executables. Implementation: Implement host integrity monitoring to detect any "
                    "unwanted altering of configuration files. Implementation: Ensure that files that "
                    "are not required to execute, such as configuration files, are not over-privileged, "
                    "i.e. not allowed to execute.",
                    "summary": "An attack of this type exploits a system's trust in configuration and resource files. "
                    "When the executable loads the resource (such as an image file or configuration file) "
                    "the attacker has modified the file to either execute malicious code directly or "
                    "manipulate the target process (e.g. application server) to execute based on the "
                    "malicious configuration parameters. Since systems are increasingly interrelated "
                    "mashing up resources from local and remote sources the possibility of this attack "
                    "occurring is high.",
                },
                {
                    "id": "77",
                    "name": "Manipulating User-Controlled Variables",
                    "prerequisites": "A variable consumed by the application server is exposed to the client. A "
                    "variable consumed by the application server can be overwritten by the user. "
                    "The application server trusts user supplied data to compute business logic. "
                    "The application server does not perform proper input validation.",
                    "related_weakness": ["15", "285", "302", "473", "94", "96"],
                    "solutions": "If the register_globals option is enabled, PHP will create global variables for "
                    "each GET, POST, and cookie variable included in the HTTP request. This means that a "
                    "malicious user may be able to set variables unexpectedly. For instance make sure "
                    "that the server setting for PHP does not expose global variables. A software system "
                    "should be reluctant to trust variables that have been initialized outside of its "
                    "trust boundary. Ensure adequate checking is performed when relying on input from "
                    "outside a trust boundary. Separate the presentation layer and the business logic "
                    "layer. Variables at the business logic layer should not be exposed at the "
                    "presentation layer. This is to prevent computation of business logic from user "
                    "controlled input data. Use encapsulation when declaring your variables. This is to "
                    "lower the exposure of your variables. Assume all input is malicious. Create a white "
                    "list that defines all valid input to the software system based on the requirements "
                    "specifications. Input that does not match against the white list should be rejected "
                    "by the program.",
                    "summary": "This attack targets user controlled variables (DEBUG=1, PHP Globals, and So Forth). "
                    "An attacker can override environment variables leveraging user-supplied, untrusted "
                    "query variables directly used on the application server without any data sanitization. "
                    "In extreme cases, the attacker can change variables controlling the business logic of "
                    "the application. For instance, in languages like PHP, a number of poorly set default "
                    "configurations may allow the user to override variables.",
                },
            ],
        ),
        "cvss": fields.Float(description="CVSS score", example=9.0),
        "impactScore": fields.Float(description="CVSS impact score", example=9.6),
        "exploitabilityScore": fields.Float(
            description="CVSS exploitability score", example=9.6
        ),
        "cvssTime": fields.DateTime(
            description="Time when the CVSS was established",
            example="2020-10-07T15:01:00",
        ),
        "cvssVector": fields.String(
            description="CVSS Vector string", example="AV:N/AC:L/Au:S/C:C/I:C/A:C"
        ),
        "cwe": fields.List(
            fields.String,
            description="CWE number to which this CVE belongs to",
            example="[CWE-94]",
        ),
        "id": fields.String(
            description="ID number of the CVE", example="CVE-2020-24365"
        ),
        "impact": fields.Nested(
            impact_entry,
            skip_none=True,
            description="Impact vectors",
            example={
                "availability": "COMPLETE",
                "confidentiality": "COMPLETE",
                "integrity": "COMPLETE",
            },
        ),
        "impact3": fields.Nested(
            impact_V3_entry,
            skip_none=True,
            description="CvssV3 impact vectors",
            example={
                "availability": "HIGH",
                "confidentiality": "HIGH",
                "integrity": "HIGH",
            },
        ),
        "impact4": fields.Nested(
            impact_V4_entry,
            skip_none=True,
            description="CvssV4" " impact vectors",
            example={
                "vulnerable_system_confidentiality": "HIGH",
                "vulnerable_system_integrity": "HIGH",
                "vulnerable_system_availability": "HIGH",
                "subsequent_system_confidentiality": "HIGH",
                "subsequent_system_integrity": "HIGH",
                "subsequent_system_availability": "HIGH",
            },
        ),
        "exploitability3": fields.Nested(
            access_V3_entry,
            skip_none=True,
            description="CvssV3 access vectors",
            example={
                "attackvector": "NETWORK",
                "attackcomplexity": "LOW",
                "privilegesrequired": "NONE",
                "userinteraction": "REQUIRED",
                "scope": "CHANGED",
            },
        ),
        "exploitability4": fields.Nested(
            access_V4_entry,
            skip_none=True,
            description="CvssV4 access vectors",
            example={
                "attackvector": "NETWORK",
                "attackcomplexity": "LOW",
                "privilegesrequired": "NONE",
                "userinteraction": "REQUIRED",
            },
        ),
        "cvss3": fields.Float(description="CvssV3 score", example=9.6),
        "cvss4": fields.Float(description="CvssV4 score", example=9.6),
        "impactScore3": fields.Float(description="CvssV3 impact score", example=9.6),
        "impactScore4": fields.Float(description="CvssV4 impact score", example=9.6),
        "exploitabilityScore3": fields.Float(
            description="CvssV3 exploitability score", example=9.6
        ),
        "exploitabilityScore4": fields.Float(
            description="CvssV4 exploitability score", example=9.6
        ),
        "cvss3Vector": fields.String(
            description="CvssV3 Vector string",
            example="CVSS:3.0/AV:N/AC:L/PR:N/UI:R/S:C/C:H/I:H/A:H",
        ),
        "cvss4Vector": fields.String(
            description="CvssV4 Vector string",
            example="CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:N/VI:N/VA:H/SC:N/SI:N/SA:N/E:P/CR:X/IR:X/AR:X/MAV:X/MAC:X/MAT:X/MPR:X/MUI:X/MVC:X/MVI:X/MVA:X/MSC:X/MSI:X/MSA:X/S:X/AU:X/R:X/V:X/RE:X/U:X",
        ),
        "lastModified": fields.DateTime(
            description="Last modified date time", example="2020-10-07T15:01:00"
        ),
        "references": fields.List(
            fields.String,
            description="CVE references",
            example=["https://pastebin.com/QTev1TjM"],
        ),
        "summary": fields.String(
            description="CVE Summary",
            example="An issue was discovered on Gemtek WRTM-127ACN 01.01.02.141 and WRTM-127x9 01.01.02.127 devices. "
            "The Monitor Diagnostic network page allows an authenticated attacker to execute a command directly "
            "on the target machine. Commands are executed as the root user (uid 0). (Even if a login is "
            "required, most routers are left with default credentials.)",
        ),
        "epss": fields.Float(
            description="EPSS probability score indicating the likelihood of exploitation within the next 30 days",
            example=0.00026,
        ),
        "epssMetric": fields.Nested(
            epss_metric,
            description="Additional EPSS metadata including percentile ranking",
        ),
    },
)

cve_entry = api.inherit(
    "CVE_Entry_Full",
    cve_entry_base,
    {
        "refmap": fields.Nested(
            refmap,
            skip_none=True,
            description="CVE Reference mapping",
            example={"misc": ["https://pastebin.com/QTev1TjM"]},
        ),
        "vulnerable_configuration": fields.List(
            fields.Nested(vuln_conf),
            skip_none=True,
            description="Vulnerable configurations CPE's",
            example=[
                {
                    "id": "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:h:gemteks:wrtm-127acn:-:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:h:gemteks:wrtm-127acn:-:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:h:gemteks:wrtm-127x9:-:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:h:gemteks:wrtm-127x9:-:*:*:*:*:*:*:*",
                },
            ],
        ),
        "vulnerable_configuration_cpe_2_2": fields.List(
            fields.Nested(vuln_conf),
            skip_none=True,
            description="Vulnerable configurations CPE's",
            example=[
                {
                    "id": "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:h:gemteks:wrtm-127acn:-:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:h:gemteks:wrtm-127acn:-:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
                },
                {
                    "id": "cpe:2.3:h:gemteks:wrtm-127x9:-:*:*:*:*:*:*:*",
                    "title": "cpe:2.3:h:gemteks:wrtm-127x9:-:*:*:*:*:*:*:*",
                },
            ],
        ),
        "vulnerable_product": fields.List(
            fields.String,
            description="Vulnerable products CPE's",
            example=[
                "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
            ],
        ),
        "oval": fields.List(
            fields.Nested(oval_entry),
            skip_none=True,
            description="List with Oval entries",
            example=[
                {
                    "accepted": "2014-06-16T04:00:06.077-04:00",
                    "class": "vulnerability",
                    "contributors": [
                        {
                            "name": "Dragos Prisaca",
                            "organization": "Symantec Corporation",
                        },
                        {"name": "Chandan S", "organization": "SecPod Technologies"},
                        {"name": "Maria Kedovskaya", "organization": "ALTX-SOFT"},
                        {"name": "Maria Mikhno", "organization": "ALTX-SOFT"},
                    ],
                    "definition_extensions": [
                        {
                            "comment": "Microsoft Windows Vista (32-bit) Service Pack 1 is installed",
                            "oval": "oval:org.mitre.oval:def:4873",
                        },
                        {
                            "comment": "Microsoft Windows Vista x64 Edition Service Pack 1 is installed",
                            "oval": "oval:org.mitre.oval:def:5254",
                        },
                        {
                            "comment": "Microsoft Windows Vista (32-bit) Service Pack 2 is installed",
                            "oval": "oval:org.mitre.oval:def:6124",
                        },
                        {
                            "comment": "Microsoft Windows Vista x64 Edition Service Pack 2 is installed",
                            "oval": "oval:org.mitre.oval:def:5594",
                        },
                        {
                            "comment": "Microsoft Windows 7 (32-bit) is installed",
                            "oval": "oval:org.mitre.oval:def:6165",
                        },
                        {
                            "comment": "Microsoft Windows 7 x64 Edition is installed",
                            "oval": "oval:org.mitre.oval:def:5950",
                        },
                        {
                            "comment": "Microsoft Windows 7 (32-bit) Service Pack 1 is installed",
                            "oval": "oval:org.mitre.oval:def:12292",
                        },
                        {
                            "comment": "Microsoft Windows 7 x64 Service Pack 1 is installed",
                            "oval": "oval:org.mitre.oval:def:12627",
                        },
                    ],
                    "description": "The Bluetooth Stack 2.1 in Microsoft Windows Vista SP1 and SP2 and Windows 7 Gold "
                    "and SP1 does not prevent access to objects in memory that (1) were not properly "
                    "initialized or (2) have been deleted, which allows remote attackers to execute "
                    'arbitrary code via crafted Bluetooth packets, aka "Bluetooth Stack Vulnerability."',
                    "family": "windows",
                    "id": "oval:org.mitre.oval:def:12094",
                    "status": "accepted",
                    "submitted": "2011-07-12T13:00:00",
                    "title": "Bluetooth Stack Vulnerability",
                    "version": "52",
                }
            ],
        ),
        "msbulletin": fields.List(
            fields.Nested(MsBulletin),
            skip_none=True,
            description="List with msbulletins",
            example=[
                {
                    "bulletin_id": "MS11-053",
                    "bulletin_url": "null",
                    "date": "2011-07-12T00:00:00",
                    "impact": "Remote Code Execution",
                    "knowledgebase_id": "2566220",
                    "knowledgebase_url": "null",
                    "severity": "Critical",
                    "title": "Vulnerability in Bluetooth Stack Could Allow Remote Code Execution",
                }
            ],
        ),
        "statements": fields.List(
            fields.Nested(statements),
            skip_none=True,
            description="List with statements",
            example=[
                {
                    "contributor": "Tomas Hoger",
                    "lastmodified": "2009-09-02",
                    "organization": "Red Hat",
                    "statement": "This issue did not affect the versions of Linux kernel as shipped with Red Hat "
                    "Enterprise Linux 2.1, 3, 4, and Red Hat Enterprise MRG. This issue was addressed "
                    "in Red Hat Enterprise Linux 5 by https://rhn.redhat.com/errata/RHSA-2009-1243.html",
                }
            ],
        ),
        "redhat": fields.Nested(
            redhat_entry,
            skip_none=True,
            description="Redhat Security advisories",
            example={
                "advisories": [{"rhsa": {"id": "RHSA-2009:1243"}}],
                "rpms": [
                    "kernel-0:2.6.18-164.el5",
                    "kernel-PAE-0:2.6.18-164.el5",
                    "kernel-PAE-debuginfo-0:2.6.18-164.el5",
                    "kernel-PAE-devel-0:2.6.18-164.el5",
                    "kernel-debug-0:2.6.18-164.el5",
                    "kernel-debug-debuginfo-0:2.6.18-164.el5",
                    "kernel-debug-devel-0:2.6.18-164.el5",
                    "kernel-debuginfo-0:2.6.18-164.el5",
                    "kernel-debuginfo-common-0:2.6.18-164.el5",
                    "kernel-devel-0:2.6.18-164.el5",
                    "kernel-doc-0:2.6.18-164.el5",
                    "kernel-headers-0:2.6.18-164.el5",
                    "kernel-kdump-0:2.6.18-164.el5",
                    "kernel-kdump-debuginfo-0:2.6.18-164.el5",
                    "kernel-kdump-devel-0:2.6.18-164.el5",
                    "kernel-xen-0:2.6.18-164.el5",
                    "kernel-xen-debuginfo-0:2.6.18-164.el5",
                    "kernel-xen-devel-0:2.6.18-164.el5",
                ],
            },
        ),
    },
)

cve_last_entries = api.inherit(
    "CVE_Entry_last",
    cve_entry_base,
    {
        "vulnerable_configuration": fields.List(
            fields.String,
            description="Vulnerable configurations CPE's",
            example=[
                "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
            ],
        ),
        "vulnerable_configuration_cpe_2_2": fields.List(
            fields.String,
            description="Vulnerable configurations CPE's",
            example=[
                "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
            ],
        ),
        "vulnerable_product": fields.List(
            fields.String,
            description="Vulnerable products CPE's",
            example=[
                "cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:*:*:*:*:*:*:*",
                "cpe:2.3:o:gemteks:wrtm-127x9_firmware:01.01.02.127:*:*:*:*:*:*:*",
            ],
        ),
    },
)

search_entries = api.model(
    "SearchEntries",
    {
        "results": fields.List(
            fields.Nested(cve_last_entries),
            description="CVE results",
            example=[
                {
                    "modified": "2018-10-12T21:29:00",
                    "published": "1999-05-07T04:00:00",
                    "access": {
                        "authentication": "NONE",
                        "complexity": "HIGH",
                        "vector": "NETWORK",
                    },
                    "assigner": "cve@mitre.org",
                    "cvss": 2.6,
                    "impactScore": 3.0,
                    "exploitabilityScore": 3.0,
                    "cvssTime": "2018-10-12T21:29:00",
                    "cvssVector": "AV:N/AC:H/Au:N/C:N/I:P/A:N",
                    "cwe": ["NVD-CWE-Other"],
                    "id": "CVE-1999-0717",
                    "impact": {
                        "availability": "NONE",
                        "confidentiality": "NONE",
                        "integrity": "PARTIAL",
                    },
                    "impact3": {
                        "availability": "HIGH",
                        "confidentiality": "HIGH",
                        "integrity": "HIGH",
                    },
                    "exploitability3": {
                        "attackvector": "NETWORK",
                        "attackcomplexity": "LOW",
                        "privilegesrequired": "NONE",
                        "userinteraction": "REQUIRED",
                        "scope": "CHANGED",
                    },
                    "cvss3": 9.6,
                    "impactScore3": 3.0,
                    "exploitabilityScore3": 6.6,
                    "cvss3Vector": "CVSS:3.0/AV:N/AC:L/PR:N/UI:R/S:C/C:H/I:H/A:H",
                    "lastModified": "2018-10-12T21:29:00",
                    "references": [
                        "http://support.microsoft.com/default.aspx?scid=kb;[LN];Q231304",
                        "https://docs.microsoft.com/en-us/security-updates/securitybulletins/1999/ms99-014",
                    ],
                    "summary": "A remote attacker can disable the virus warning mechanism in Microsoft Excel 97.",
                    "vulnerable_configuration": [
                        "cpe:2.3:a:microsoft:excel:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_2000:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_95:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_98:*:gold:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_nt:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_nt:4.0:*:*:*:*:*:*:*",
                    ],
                    "vulnerable_configuration_cpe_2_2": [],
                    "vulnerable_product": [
                        "cpe:2.3:a:microsoft:excel:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_2000:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_95:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_98:*:gold:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_nt:*:*:*:*:*:*:*:*",
                        "cpe:2.3:o:microsoft:windows_nt:4.0:*:*:*:*:*:*:*",
                    ],
                },
                {
                    "modified": "2018-10-12T21:29:00",
                    "published": "1999-10-01T04:00:00",
                    "access": {
                        "authentication": "NONE",
                        "complexity": "LOW",
                        "vector": "LOCAL",
                    },
                    "assigner": "cve@mitre.org",
                    "cvss": 4.6,
                    "impactScore": 3.0,
                    "exploitabilityScore": 3.0,
                    "cvssTime": "2018-10-12T21:29:00",
                    "cvssVector": "AV:L/AC:L/Au:N/C:P/I:P/A:P",
                    "cwe": ["CWE-59"],
                    "id": "CVE-1999-0794",
                    "impact": {
                        "availability": "PARTIAL",
                        "confidentiality": "PARTIAL",
                        "integrity": "PARTIAL",
                    },
                    "impact3": {
                        "availability": "HIGH",
                        "confidentiality": "HIGH",
                        "integrity": "HIGH",
                    },
                    "exploitability3": {
                        "attackvector": "NETWORK",
                        "attackcomplexity": "LOW",
                        "privilegesrequired": "NONE",
                        "userinteraction": "REQUIRED",
                        "scope": "CHANGED",
                    },
                    "cvss3": 9.6,
                    "cvss4": 9.6,
                    "impactScore3": 3.0,
                    "impactScore4": 3.0,
                    "exploitabilityScore3": 6.6,
                    "exploitabilityScore4": 6.6,
                    "cvss3Vector": "CVSS:3.0/AV:N/AC:L/PR:N/UI:R/S:C/C:H/I:H/A:H",
                    "cvss4Vector": "CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:N/VI:N/VA:H/SC:N/SI:N/SA:N/E:P/CR:X/IR:X/AR:X/MAV:X/MAC:X/MAT:X/MPR:X/MUI:X/MVC:X/MVI:X/MVA:X/MSC:X/MSI:X/MSA:X/S:X/AU:X/R:X/V:X/RE:X/U:X",
                    "lastModified": "2018-10-12T21:29:00",
                    "references": [
                        "http://support.microsoft.com/default.aspx?scid=kb;[LN];Q241900",
                        "http://support.microsoft.com/default.aspx?scid=kb;[LN];Q241901",
                        "http://support.microsoft.com/default.aspx?scid=kb;[LN];Q241902",
                        "https://docs.microsoft.com/en-us/security-updates/securitybulletins/1999/ms99-044",
                    ],
                    "summary": "Microsoft Excel does not warn a user when a macro is present in a Symbolic Link "
                    "(SYLK) format file.",
                    "vulnerable_configuration": [
                        "cpe:2.3:a:microsoft:excel:*:*:*:*:*:*:*:*",
                        "cpe:2.3:a:microsoft:office:*:*:*:*:*:*:*:*",
                    ],
                    "vulnerable_configuration_cpe_2_2": [],
                    "vulnerable_product": ["cpe:2.3:a:microsoft:excel:*:*:*:*:*:*:*:*"],
                },
            ],
        ),
        "total": fields.Integer(
            description="Total amount of records matching this search criteria",
            example="1337",
        ),
    },
)


@api.route("/cve/<cveid>")
@api.param("cveid", "CVE number", example="CVE-2016-3333", required=True)
@api.response(400, "Error processing request", model=message)
@api.response(404, "The requested CVE is not found", model=message)
@api.response(500, "Server error", model=message)
class CveId(Resource):
    @api.marshal_with(cve_entry, skip_none=True)
    def get(self, cveid):
        """
        CVE from CVE ID

        Outputs all available information for the specified CVE (Common Vulnerability and Exposure), in JSON format.
        This information includes basic CVE information like CVSS (Common Vulnerability Scoring System),
        related CPE (Common Product Enumeration), CWE (Common Weakness Enumeration), etc as well as additional
        information (RedHat Advisories etc).
        """
        cvesp = CveHandler(
            rankinglookup=True, namelookup=True, via4lookup=True, capeclookup=True
        )
        cve = cvesp.getcve(cveid=cveid.upper())
        if cve is None:
            api.abort(404, "The requested CVE is not found")
        else:
            return cve


@api.route("/cvefor/<path:cpe>")
@api.param(
    "cpe",
    "CPE code. Accepts both CPE v2.2 URI & CPE v2.3 formatted string",
    example="cpe:2.3:o:gemteks:wrtm-127acn_firmware:01.01.02.141:\*:\*:\*:\*:\*:\*:\*",
    required=True,
)
@api.param(
    "limit",
    "Limit the amount of returned documents. (Regardless the value, the maximum amount of documents comes from configuration parameter API:CVEMaxLimit.)",
    example=10,
    default=500,
    _in="query",
)
@api.response(400, "Error processing request", model=message)
@api.response(404, "No cves found", model=message)
@api.response(500, "Server error", model=message)
class CveFor(Resource):
    @api.marshal_list_with(cve_entry, skip_none=True)
    def get(self, cpe, set_limit=500):
        """
        CVE's from CPE ID

        Outputs a list of CVEs related to the product
        """
        limit = request.args.get("limit", None)

        if limit is None:
            limit = set_limit
        elif int(limit) < int(config.getCVEMaxLimit()):
            limit = int(limit)
        else:
            limit = int(config.getCVEMaxLimit())

        cves = qcvesForCPE(cpe, int(limit))
        if len(cves) == 0:
            api.abort(404, "No cves found")
        else:
            return cves


@api.route("/last")
@api.param(
    "limit",
    "Limit the amount of returned documents. (Regardless the value, the maximum amount of documents comes from configuration parameter API:CVEMaxLimit.)",
    example=10,
    default=30,
    _in="query",
)
@api.response(400, "Error processing request", model=message)
@api.response(404, "No cves found", model=message)
@api.response(500, "Server error", model=message)
class CveLast(Resource):
    @api.marshal_list_with(cve_last_entries, skip_none=True)
    def get(self, set_limit=30):
        """
        List last CVE's

        Outputs the last n amount of vulnerabilities. If the limit is not specified, the default of 30 is used.
        """
        limit = request.args.get("limit", None)

        if limit is None:
            limit = set_limit
        elif int(limit) < int(config.getCVEMaxLimit()):
            limit = int(limit)
        else:
            limit = int(config.getCVEMaxLimit())

        cvesp = CveHandler(
            rankinglookup=True, namelookup=True, via4lookup=True, capeclookup=True
        )
        cve = cvesp.get(limit=limit)

        if len(cve) == 0:
            api.abort(404, "No cves found")
        else:
            return cve


@api.route("/search/<vendor>/<path:product>")
@api.param("vendor", "Vendor name", example="microsoft", required=True)
@api.param("product", "Product name", example="excel", required=True)
@api.response(400, "Error processing request", model=message)
@api.response(404, "No cves found", model=message)
@api.response(500, "Server error", model=message)
class SearchCve(Resource):
    @api.marshal_with(search_entries)
    def get(self, vendor, product):
        """
        Search CVE by vendor

        When vendor and product are specified, this API call returns a list of CVEs related to the product.
        The output of the browse call can be used for this.
        """
        from lib.DatabaseLayer import cvesForCPE

        search = (vendor, product)

        return cvesForCPE(search, strict_vendor_product=True)



from flask_restx import Namespace, fields, Resource

from web.restapi.cpe_convert import message

api = Namespace(
    "dbinfo",
    description="Endpoints displaying information about the cve search database",
    path="/",
)

database_inner = api.model(
    "DBInner",
    {
        "last_update": fields.DateTime(
            description="Date time of last update", example="2019-09-30T18:53:48"
        ),
        "size": fields.Integer(
            description="Amount of documents in collection", example="570"
        ),
    },
)

database_wild = fields.Wildcard(
    fields.Nested(database_inner),
    description="Database collection mapping id",
    skip_none=True,
)

database_entry = api.model("documents", {"*": database_wild})


@api.route("/dbinfo")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class DBInfo(Resource):
    @api.marshal_with(database_entry)
    def get(self):
        """
        Get Database info

        Returns the stats of the database. When the user authenticates, more information is returned.

        This information includes:
        <ul>
          <li> Some server settings like the database name </li>
          <li> Some database information like disk usage </li>
        </ul>
        Unauthenticated queries return only collection information.
        """
        from lib.DatabaseLayer import getDBStats

        return getDBStats()



import pytest

from test.runners.cli_test_runner import CLITestRunner


@pytest.fixture
def runner():
    return CLITestRunner()


def test_search_cpe(runner):
    result = runner.runcommand("bin/search_cpe.py -s wordpress")

    resultlist = result.stdout.split("\n")

    assert result.returncode == 0

    # check if wordpress is matched in returned results; first, somewhere middle and somewhere last
    first_val = resultlist[0][41:].lower()
    middle_val = resultlist[int(len(resultlist) / 2)][41:].lower()
    last_val = resultlist[-1][41:].lower()
    if last_val == "":
        last_val = resultlist[-2][41:].lower()

    assert "wordpress" in first_val
    assert "wordpress" in middle_val
    assert "wordpress" in last_val


def test_search_cpe_json(runner):
    result = runner.runcommand("bin/search_cpe.py -s wordpress -o json")

    assert result.returncode == 0


def test_search_cpe_compact(runner):
    result = runner.runcommand("bin/search_cpe.py -s wordpress -o compact")

    assert result.returncode == 0


def test_search_cpe_csv(runner):
    result = runner.runcommand("bin/search_cpe.py -s wordpress -o csv")

    assert result.returncode == 0



import calendar
from datetime import datetime

date_time_formats = [
    "%d-%m-%Y",
    "%Y-%m-%d",
    "%d-%m-%Y %H:%M",
    "%Y-%m-%d %H:%M:%S",
    "%H:%M:%S %d-%m-%Y",
    "%Y-%m-%dT%H:%M:%S",
    "%Y-%m-%dT%H:%M:%SZ",
    "%Y-%m-%dT%H:%M:%S.%f",
    "%Y-%m-%dT%H:%M:%S,%f",
    "%Y-%m-%dT%H:%M:%S.%fZ",
    "%Y-%m-%dT%H:%M:%S,%fZ",
]


def timestringTOdatetime(timestring):
    """
    Method to convert a given date time string into a datetime object. Timestring is matched to the date_time_formats
    'date time string formats' list. If matched will return a datetime object; will return False otherwise.

    :param timestring: date time string
    :type timestring: str
    :return: Datetime object
    :rtype: datetime
    """

    match = False

    # try to match string formats to given string
    for each in date_time_formats:
        try:
            match = datetime.strptime(timestring, each)
        except ValueError:
            continue

    return match


def datetimeTOtimestamp(date_time_object):
    """
    Method that will take the provided date time and converts it into a timestamp

    :param date_time_object: date time object
    :type date_time_object: datetime
    :return: unix timestamp
    :rtype: int
    """

    return calendar.timegm(date_time_object.utctimetuple())


def timestampTOdatetime(timestamp):
    """
    Method that will take the provided timestamp and converts it into a date time object

    :param timestamp: unix timestamp
    :type timestamp: int
    :return: date time object
    :rtype: datetime
    """
    value = datetime.utcfromtimestamp(timestamp)

    return value



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Admin creator script
#
# Creates an admin account in the database
# Only master accounts are allowed to add and remove users
# First account registered is the master account
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import getpass
import os
import sys
from hmac import compare_digest

from pymongo.errors import ConnectionFailure

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.DatabaseHandler import DatabaseHandler
from lib.DatabaseLayer import getSize

dbh = DatabaseHandler()

# args
argParser = argparse.ArgumentParser(
    description="Admin account creator for the mongo database"
)
argParser.add_argument("-a", help="<name> Add an account", default=False)
argParser.add_argument("-c", help="Change the password of an account", default=None)
argParser.add_argument("-r", help="Remove account", default=False)
argParser.add_argument("-p", help="Promote account to master", default=False)
argParser.add_argument("-d", help="Demote account to normal user", default=False)
argParser.add_argument("-l", help="Make the user local-only", action="store_true")
args = argParser.parse_args()

# vars
col = "mgmt_users"

exits = {
    "userInDb": "User already exists in database",
    "userNotInDb": "User does not exist in database",
    "userpasscombo": "Master user/password combination does not exist",
    "passwordMatch": "The passwords don't match!",
    "noMaster": "Not a master account!",
    "lastMaster": "This user is the last admin in the database and thus can not be removed",
    "dummy": "_dummy_ is a placeholder, and thus cannot be used",
}

# functions


def verifyPass(password, user):
    if not dbh.connection.userExists(user):
        sys.exit(exits["userNotInDb"])
    if not dbh.connection.verifyUser(user, password):
        sys.exit(exits["userpasscombo"])
    return True


def promptNewPass():
    password = getpass.getpass("New password:")
    verify = getpass.getpass("Verify password:")
    if not compare_digest(password, verify):
        sys.exit(exits["passwordMatch"])
    return password


def masterLogin():
    master = input("Master account username: ")
    if verifyPass(getpass.getpass("Master password:"), master):
        if not dbh.connection.isMasterAccount(master):
            sys.exit(exits["noMaster"])
    else:
        sys.exit("Master user/password combination does not exist")
    return True


def isLastAdmin(user):
    if dbh.connection.isSingleMaster(user):
        sys.exit(exits["lastMaster"])


# script run
try:
    if args.a:
        username = args.a
        if username.strip() == "_dummy_":
            sys.exit(exits["dummy"])
        if dbh.connection.userExists(username):
            sys.exit(exits["userInDb"])
        # set master if db is empty
        if getSize(col) > 0:
            masterLogin()
            password = promptNewPass()
            dbh.connection.addUser(username, password, localOnly=args.l)
        else:
            password = promptNewPass()
            dbh.connection.addUser(username, password, admin=True, localOnly=args.l)
        sys.exit("User added")
    elif args.c:
        username = args.c
        verifyPass(getpass.getpass("Old password:"), username)
        password = promptNewPass()
        dbh.connection.changePassword(username, password)
        sys.exit("Password updated")
    elif args.r:
        username = args.r
        if not dbh.connection.userExists(username):
            sys.exit(exits["userNotInDb"])
        masterLogin()
        isLastAdmin(username)
        dbh.connection.deleteUser(username)
        sys.exit("User removed from database")
    elif args.p:
        username = args.p
        if not dbh.connection.userExists(username):
            sys.exit(exits["userNotInDb"])
        masterLogin()
        # promote
        dbh.connection.setAdmin(username, True)
        sys.exit("User promoted")
    elif args.d:
        username = args.d
        if not dbh.connection.userExists(username):
            sys.exit(exits["userNotInDb"])
        masterLogin()
        isLastAdmin(username)
        # demote
        dbh.connection.setAdmin(username, False)
        sys.exit("User demoted")

except ConnectionFailure:
    print("Can't connect to the mongo database")
except Exception as e:
    print(e)



import logging
from abc import ABC, abstractmethod
from datetime import datetime

from nested_lookup import nested_lookup, nested_update

from lib.LogHandler import AppLogger
from web.helpers.common import timestringTOdatetime, date_time_formats

logging.setLoggerClass(AppLogger)


def convertDatetime(dct=None):
    if isinstance(dct, (list, tuple, set)):
        for item in dct:
            convertDatetime(item)
    elif type(dct) is dict:
        for key, val in dct.items():
            if isinstance(val, datetime):
                dct[key] = val.isoformat()
            if isinstance(val, (dict, list)):
                convertDatetime(val)
    return dct


class ApiRequest(ABC):
    def __init__(self):
        self._request_results = None

    @abstractmethod
    def process(self, **kwargs):
        raise NotImplementedError

    @property
    def request_results(self):
        return self._request_results

    @request_results.setter
    def request_results(self, the_results):
        self._request_results = {"total": len(the_results), "data": the_results}


class JSONApiRequest(ApiRequest):
    def __init__(self, headers, body):
        """
        Main class for the JSONApiRequest endpoints of the api.

        :param headers: Request headers
        :type headers: dict
        :param body: JSON query body
        :type body: dict
        """
        super().__init__()
        self.logger = logging.getLogger(__name__)

        self.request_headers = headers
        self.body = body

    def process(self, database_connection):
        """
        Method to process the request

        :param database_connection: Hookup to the database
        :type database_connection: DatabaseHandler
        :return: Results from the request
        :rtype: list
        """
        self.logger.debug(
            "Processing request: {}  Headers received: {}".format(
                self.body, self.request_headers
            )
        )

        validated, reason = self.validate_body()

        if not validated:
            self.logger.warning(
                "Validation on {} not succeeded: {}".format(self.body, reason)
            )
            return reason, 400
        else:
            results = database_connection.query_docs(**self.body)

            self.logger.debug(
                "Retrieving from: {} -- records: {}".format(
                    self.body["retrieve"], len(results)
                )
            )

            self.request_results = results

            return convertDatetime(dct=self.request_results)

    def validate_body(self):
        """
        Method for validating the request body.

        """
        retrieve = ["capec", "cpe", "cves", "cwe", "via4"]

        sort_dir = ["ASC", "DESC"]

        required_keys = ["retrieve", "dict_filter"]

        optional_keys = ["sort", "limit", "skip", "query_filter", "sort_dir"]

        date_keys = ["published", "modified", "lastModified"]

        if self.body is None or len(self.body) == 0:
            return (
                False,
                "Please send a proper request with a json formatted like in the documentation.",
            )

        if not all(key.lower() in self.body for key in required_keys):
            return False, "Request is missing one or more required keys!"

        # convert date_keys from string to datetime objects
        if "dict_filter" in self.body:
            for key in date_keys:
                convert_values = nested_lookup(key, self.body["dict_filter"])
                if len(convert_values) != 0:
                    converted_val = self.convert_datetimestring_to_datetime(
                        convert_values[0]
                    )
                    if converted_val:
                        nested_update(
                            document=self.body["dict_filter"],
                            key=key,
                            value=converted_val,
                        )
                    else:
                        return (
                            False,
                            "Could not convert provided date string to valid datetime object; currently supported "
                            "formats are: {}".format(date_time_formats),
                        )

        if not self.body["retrieve"].lower() in retrieve:
            return False, "Unable to retrieve from specified data source!"

        if "sort_dir" in self.body.keys():
            if not self.body["sort_dir"] in sort_dir:
                return (
                    False,
                    "Specified sorting direction not possible; possible options are: {}!".format(
                        sort_dir
                    ),
                )

        all_keys = retrieve + required_keys + optional_keys

        if not all(key in all_keys for key in self.body):
            return False, "Request contains unknown keys!"

        if "skip" in self.body:
            try:
                self.body["skip"] = int(self.body["skip"])
            except ValueError:
                return False, "Skip parameter is not a integer!"

        if "limit" in self.body:
            try:
                self.body["limit"] = int(self.body["limit"])
            except ValueError:
                return False, "Limit parameter is not a integer!"

        return True, "Ok"

    def convert_datetimestring_to_datetime(self, dict_entry):
        """

        :param dict_entry:
        :type dict_entry: dict
        :return:
        :rtype:
        """
        date_item = False

        if isinstance(dict_entry, str):
            date_item = timestringTOdatetime(dict_entry)

        elif isinstance(dict_entry, dict):
            for k, v in dict_entry.items():
                convert_val = timestringTOdatetime(v)
                if convert_val:
                    dict_entry[k] = convert_val
                else:
                    return False

            date_item = dict_entry

        return date_item



from lib.DatabaseHandler import DatabaseHandler


class LazyDatabaseHandler:
    """
    Lazily instantiate DatabaseHandler to defer MongoClient creation until first use.
    This avoids PyMongo warnings when using prefork servers like Gunicorn.
    """

    def __init__(self, **kwargs):
        self._kwargs = kwargs
        self._real = None

    def _get_real(self):
        if self._real is None:
            self._real = DatabaseHandler(**self._kwargs)
        return self._real

    def __getattr__(self, name):
        return getattr(self._get_real(), name)

    def __setattr__(self, name, value):
        if name in ("_real", "_kwargs"):
            super().__setattr__(name, value)
        else:
            setattr(self._get_real(), name, value)


class FlaskDatabaseHandler:
    """
    Flask helper to attach a lazy DatabaseHandler to `app.dbh`.
    """

    def __init__(self, app=None, **kwargs):
        self._lazy_dbh = LazyDatabaseHandler(**kwargs)
        if app is not None:
            self.init_app(app)

    def init_app(self, app, **kwargs):
        # Merge extra kwargs and attach lazy db handler
        self._lazy_dbh._kwargs.update(kwargs)
        app.dbh = self._lazy_dbh

    def __repr__(self):
        return "<< FlaskDatabaseHandler >>"



import os
import pytest
from lib.Config import Configuration

test_cases = [
    pytest.param(
        {
            "username": "user",
            "password": "pass",
            "dns_srv": "True",
            "expected_uri": "mongodb+srv://user:pass@localhost/testdb?authSource=admin&retryWrites=true&w=majority",
            "expected_dbapi": "srv",
        },
        id="srv_with_auth",
    ),
    pytest.param(
        {
            "username": "user",
            "password": "pass",
            "dns_srv": "False",
            "expected_uri": "mongodb://user:pass@localhost:27017/testdb?authSource=admin",
            "expected_dbapi": None,
        },
        id="no_srv_with_auth",
    ),
    pytest.param(
        {
            "username": "",
            "password": "",
            "dns_srv": "True",
            "expected_uri": "mongodb+srv://localhost/testdb?retryWrites=true&w=majority",
            "expected_dbapi": "srv",
        },
        id="srv_no_auth",
    ),
    pytest.param(
        {
            "username": "",
            "password": "",
            "dns_srv": "False",
            "expected_uri": "mongodb://localhost:27017/testdb",
            "expected_dbapi": None,
        },
        id="no_srv_no_auth",
    ),
]

ENV_VARS = [
    "DATASOURCE_HOST",
    "DATASOURCE_PORT",
    "DATASOURCE_DBNAME",
    "DATASOURCE_DBAPI",
    "DATASOURCE_USER",
    "DATASOURCE_PASSWORD",
]


def _init_config(case, monkeypatch):
    """Helper to set up Configuration and clear environment for a test case."""
    # Clear environment
    for var in ENV_VARS:
        monkeypatch.delenv(var, raising=False)

    # Use real DB settings for tests that only verify configuration
    monkeypatch.delenv("USE_TEST_DB", raising=False)

    # Ensure Database section exists
    if not Configuration.ConfigParser.has_section("Database"):
        Configuration.ConfigParser.add_section("Database")

    # Set test values in the class parser
    Configuration.ConfigParser.set("Database", "Host", "localhost")
    Configuration.ConfigParser.set("Database", "Port", "27017")
    Configuration.ConfigParser.set("Database", "DB", "testdb")
    Configuration.ConfigParser.set("Database", "Username", case["username"])
    Configuration.ConfigParser.set("Database", "Password", case["password"])
    Configuration.ConfigParser.set("Database", "DnsSrvRecord", case["dns_srv"])
    Configuration.ConfigParser.set("Database", "AuthDB", "admin")


@pytest.mark.parametrize("case", test_cases)
def test_mongodb_config_url_generation(case, monkeypatch):
    """Tests that MongoDB URLs are built correctly for internal use."""
    _init_config(case, monkeypatch)
    uri = Configuration.getMongoUri()
    assert uri == case["expected_uri"]


@pytest.mark.parametrize("case", test_cases)
def test_mongodb_config_envvar_passing(case, monkeypatch):
    """Tests that MongoDB configuration is properly passed via environment variables."""
    _init_config(case, monkeypatch)
    Configuration.setMongoDBEnv()

    assert os.environ["DATASOURCE_HOST"] == "localhost"
    assert os.environ["DATASOURCE_PORT"] == "27017"
    assert os.environ["DATASOURCE_DBNAME"] == "testdb"

    if case["expected_dbapi"]:
        assert os.environ["DATASOURCE_DBAPI"] == case["expected_dbapi"]
    else:
        assert "DATASOURCE_DBAPI" not in os.environ

    if case["username"] and case["password"]:
        assert os.environ["DATASOURCE_USER"] == case["username"]
        assert os.environ["DATASOURCE_PASSWORD"] == case["password"]
    else:
        assert "DATASOURCE_USER" not in os.environ
        assert "DATASOURCE_PASSWORD" not in os.environ



import os
import sys

from flask import Flask
from werkzeug.middleware.dispatcher import DispatcherMiddleware

_runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(_runPath, ".."))

from lib.Config import Configuration
from web.run import create_app
from web.set_version import _version

__version__ = _version()

config = Configuration()

app = Flask(__name__)
cveapp = create_app(__version__, _runPath)

app.wsgi_app = DispatcherMiddleware(Flask("FRAME"), {config.getMountPath(): cveapp})


if __name__ == "__main__":
    app.run(host="0.0.0.0")



import json

from bson import json_util
from dicttoxml import dicttoxml
from flask import request, Response
from flask_restx import Namespace, fields, Resource

from lib.ApiRequests import JSONApiRequest
from lib.DatabaseHandler import DatabaseHandler
from web.home.utils import filter_logic, parse_headers
from web.restapi.cpe_convert import message
from web.restapi.cve import cve_last_entries

api = Namespace(
    "query", description="Endpoints for querying the cve search database", path="/"
)


query_entries = api.model(
    "QueryEntries",
    {
        "results": fields.List(
            fields.Nested(cve_last_entries),
            description="Results from query",
        ),
        "total": fields.Integer(
            description="Total amount of records available for this query",
            example="1337",
        ),
    },
)

PostBodyRequest = api.model(
    "ApiPostRequest",
    {
        "retrieve": fields.String(
            description="Retrieve data from this collection, allowed options are 'capec', 'cpe', 'cves', 'cwe', 'via4'",
            example="cves",
            required=True,
        ),
        "dict_filter": fields.Raw(
            description="filter according to pymongo docs",
            example={
                "vulnerable_configuration": "cpe:2.3:o:totolink:t10_firmware:4.1.8cu.5803_b20200521:*:*:*:*:*:*:*"
            },
            required=True,
        ),
        "limit": fields.Integer(
            description="Limit the amount of returned documents", example=10
        ),
        "skip": fields.Integer(
            description="Skip the first N amount of documents", example=0
        ),
        "sort": fields.String(description="Sort on this field", example="cvss"),
        "sort_dir": fields.String(
            description="sorting direction ASC = pymongo.ASCENDING, DESC = pymongo.DESCENDING",
            example="ASC",
        ),
        "query_filter": fields.Raw(
            description="query filter to exclude certain fields (via a 0) or to limit query to a specific set (via a 1)",
            example={"access": 0, "cwe": 0},
        ),
    },
)

PostBodyResponse = api.model(
    "ApiPostResponse",
    {
        "data": fields.List(
            fields.Raw,
            description="Returned data",
            example=[
                {
                    "_id": "695013ba7c7c55912d065a6d",
                    "access": {
                        "authentication": "NONE",
                        "complexity": "LOW",
                        "vector": "NETWORK",
                    },
                    "assigner": "cna@vuldb.com",
                    "configurations": [
                        {
                            "nodes": [
                                {
                                    "cpeMatch": [
                                        {
                                            "criteria": "cpe:2.3:o:totolink:t10_firmware:4.1.8cu.5803_b20200521:*:*:*:*:*:*:*",
                                            "matchCriteriaId": "47CB81FA-1CCC-4BE4-A16B-DBAC90971DC1",
                                            "vulnerable": True,
                                        }
                                    ],
                                    "negate": False,
                                    "operator": "OR",
                                },
                                {
                                    "cpeMatch": [
                                        {
                                            "criteria": "cpe:2.3:h:totolink:t10:-:*:*:*:*:*:*:*",
                                            "matchCriteriaId": "1C410805-E3D4-4F8C-8DF4-013ACE3937FA",
                                            "vulnerable": False,
                                        }
                                    ],
                                    "negate": False,
                                    "operator": "OR",
                                },
                            ],
                            "operator": "AND",
                        }
                    ],
                    "cvss": 10,
                    "cvss3": 9.8,
                    "cvss3Source": "cna@vuldb.com",
                    "cvss3Time": "2025-12-30T18:16:07.343000",
                    "cvss3Type": "Primary",
                    "cvss3Vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                    "cvss4": 9.3,
                    "cvss4Source": "cna@vuldb.com",
                    "cvss4Time": "2025-12-30T18:16:07.343000",
                    "cvss4Type": "Secondary",
                    "cvss4Vector": "CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:H/VI:H/VA:H/SC:N/SI:N/SA:N/E:X/CR:X/IR:X/AR:X/MAV:X/MAC:X/MAT:X/MPR:X/MUI:X/MVC:X/MVI:X/MVA:X/MSC:X/MSI:X/MSA:X/S:X/AU:X/R:X/V:X/RE:X/U:X",
                    "cvssSource": "cna@vuldb.com",
                    "cvssTime": "2025-12-30T18:16:07.343000",
                    "cvssType": "Secondary",
                    "cvssVector": "AV:N/AC:L/Au:N/C:C/I:C/A:C",
                    "cvss_data": {
                        "cvss2": {
                            "cna@vuldb.com": [
                                {
                                    "accessComplexity": "LOW",
                                    "accessVector": "NETWORK",
                                    "authentication": "NONE",
                                    "availabilityImpact": "COMPLETE",
                                    "baseScore": 10,
                                    "confidentialityImpact": "COMPLETE",
                                    "integrityImpact": "COMPLETE",
                                    "type": "Secondary",
                                    "vectorString": "AV:N/AC:L/Au:N/C:C/I:C/A:C",
                                }
                            ]
                        },
                        "cvss3": {
                            "cna@vuldb.com": [
                                {
                                    "attackComplexity": "LOW",
                                    "attackVector": "NETWORK",
                                    "availabilityImpact": "HIGH",
                                    "baseScore": 9.8,
                                    "confidentialityImpact": "HIGH",
                                    "integrityImpact": "HIGH",
                                    "privilegesRequired": "NONE",
                                    "scope": "UNCHANGED",
                                    "type": "Primary",
                                    "userInteraction": "NONE",
                                    "vectorString": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
                                }
                            ]
                        },
                        "cvss4": {
                            "cna@vuldb.com": [
                                {
                                    "attackComplexity": "LOW",
                                    "attackRequirements": "NONE",
                                    "attackVector": "NETWORK",
                                    "baseScore": 9.3,
                                    "exploitMaturity": "NOT_DEFINED",
                                    "privilegesRequired": "NONE",
                                    "subsequent_system_availability": "NONE",
                                    "subsequent_system_confidentiality": "NONE",
                                    "subsequent_system_integrity": "NONE",
                                    "type": "Secondary",
                                    "userInteraction": "NONE",
                                    "vectorString": "CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:H/VI:H/VA:H/SC:N/SI:N/SA:N/E:X/CR:X/IR:X/AR:X/MAV:X/MAC:X/MAT:X/MPR:X/MUI:X/MVC:X/MVI:X/MVA:X/MSC:X/MSI:X/MSA:X/S:X/AU:X/R:X/V:X/RE:X/U:X",
                                    "vulnerable_system_availability": "HIGH",
                                    "vulnerable_system_confidentiality": "HIGH",
                                    "vulnerable_system_integrity": "HIGH",
                                }
                            ]
                        },
                    },
                    "cwe": ["CWE-119", "CWE-121"],
                    "epss": "0.00101",
                    "epssMetric": {
                        "lastModified": "2025-12-30T13:31:06",
                        "percentile": "0.28594",
                    },
                    "exploitability3": {
                        "attackcomplexity": "LOW",
                        "attackvector": "NETWORK",
                        "privilegesrequired": "NONE",
                        "scope": "UNCHANGED",
                        "userinteraction": "NONE",
                    },
                    "exploitability4": {
                        "attackcomplexity": "LOW",
                        "attackrequirements": "NONE",
                        "exploitmaturity": "NOT_DEFINED",
                        "privilegesrequired": "NONE",
                        "userinteraction": "NONE",
                    },
                    "exploitabilityScore": 10,
                    "exploitabilityScore3": 3.9,
                    "id": "CVE-2025-14964",
                    "impact": {
                        "availability": "COMPLETE",
                        "confidentiality": "COMPLETE",
                        "integrity": "COMPLETE",
                    },
                    "impact3": {
                        "availability": "HIGH",
                        "confidentiality": "HIGH",
                        "integrity": "HIGH",
                    },
                    "impact4": {
                        "attackvector": "NETWORK",
                        "subsequent_system_availability": "NONE",
                        "subsequent_system_confidentiality": "NONE",
                        "subsequent_system_integrity": "NONE",
                        "vulnerable_system_availability": "HIGH",
                        "vulnerable_system_confidentiality": "HIGH",
                        "vulnerable_system_integrity": "HIGH",
                    },
                    "impactScore": 10,
                    "impactScore3": 5.9,
                    "lastModified": "2025-12-30T18:16:07.343000",
                    "modified": "2025-12-30T18:16:07.343000",
                    "products": ["t10_firmware"],
                    "published": "2025-12-19T19:15:50.213000",
                    "references": [
                        "https://github.com/JackWesleyy/CVE/blob/main/TOTOLINK_T10_BOC.md",
                        "https://vuldb.com/?ctiid.337599",
                        "https://vuldb.com/?id.337599",
                        "https://vuldb.com/?submit.717720",
                        "https://www.totolink.net/",
                    ],
                    "status": "Analyzed",
                    "summary": "A vulnerability has been found in TOTOLINK T10 4.1.8cu.5083_B20200521. This affects the function sprintf of the file /cgi-bin/cstecgi.cgi. Such manipulation of the argument loginAuthUrl leads to stack-based buffer overflow. The attack may be performed from remote.",
                    "vendors": ["totolink"],
                    "vulnerable_configuration": [
                        "cpe:2.3:o:totolink:t10_firmware:4.1.8cu.5803_b20200521:*:*:*:*:*:*:*",
                        "cpe:2.3:h:totolink:t10:-:*:*:*:*:*:*:*",
                    ],
                    "vulnerable_configuration_cpe_2_2": [],
                    "vulnerable_configuration_stems": [
                        "cpe:2.3:o:totolink:t10_firmware",
                        "cpe:2.3:h:totolink:t10",
                    ],
                    "vulnerable_product": [
                        "cpe:2.3:o:totolink:t10_firmware:4.1.8cu.5803_b20200521:*:*:*:*:*:*:*"
                    ],
                    "vulnerable_product_stems": ["cpe:2.3:o:totolink:t10_firmware"],
                }
            ],
        ),
        "total": fields.Integer(
            description="Total amount of returned documents", example=1
        ),
    },
)


@api.route("/query")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class QueryApi(Resource):
    @api.param(
        "rejected",
        "Hide or show rejected CVEs",
        example="hide/show",
        default="show",
        _in="header",
    )
    @api.param("cvss", "CVSS score", example=6.8, _in="header")
    @api.param(
        "cvssSelect",
        "Select the CVSS score of the CVEs related to cvss_score",
        example="above/equals/below",
        _in="header",
    )
    @api.param(
        "cvssVersion",
        "Select which version of CVSS needs to be queried",
        example="V2/V3",
        default="V3",
        _in="header",
    )
    @api.param(
        "startDate",
        "Earliest time for a CVE",
        example="dd-mm-yyyy or dd-mm-yy format, using - or /",
        _in="header",
    )
    @api.param(
        "endDate",
        "Latest time for a CVE",
        example="dd-mm-yyyy or dd-mm-yy format, using - or /",
        _in="header",
    )
    @api.param(
        "timeSelect",
        "Timeframe for the CVEs, related to the start and end time",
        example="from/until/between/outside",
        _in="header",
    )
    @api.param(
        "timeTypeSelect",
        "Select which time is used for the filter",
        example="modified/published/lastModified",
        _in="header",
    )
    @api.param(
        "skip", "Skip the n latest vulnerabilities", example=50, _in="header", type=int
    )
    @api.param(
        "limit",
        "Limit the amount of vulnerabilities to return",
        example=20,
        _in="header",
        type=int,
    )
    @api.marshal_with(query_entries, skip_none=True)
    def get(self):
        """
        Query for CVE's

        Returns a list of CVEs matching the criteria of the filters specified in the headers.
        """
        f = {
            "rejectedSelect": request.headers.get("rejected"),
            "cvss": request.headers.get("cvss"),
            "cvssSelect": request.headers.get("cvssSelect"),
            "cvssVersion": request.headers.get("cvssVersion"),
            "startDate": request.headers.get("startDate"),
            "endDate": request.headers.get("endDate"),
            "timeSelect": request.headers.get("timeSelect"),
            "timeTypeSelect": request.headers.get("timeTypeSelect"),
            "skip": request.headers.get("skip"),
            "limit": request.headers.get("limit"),
        }

        try:
            skip = int(f["skip"]) if f["skip"] else 0
        except ValueError:
            api.abort(400, "Skip variable should be an integer")

        try:
            limit = int(f["limit"]) if f["limit"] else 0
        except ValueError:
            api.abort(400, "Limit variable should be an integer")

        results = filter_logic(f, skip, limit)

        if len(results) == 0:
            api.abort(404, "")
        else:
            return results

    @api.doc(body=PostBodyRequest)
    @api.response(200, "OK", PostBodyResponse)
    @api.param(
        "format",
        "Specify in which format the results must be returned",
        example="json/xml",
        default="json",
        _in="query",
    )
    def post(self, format_output="json"):
        """
        Free query

        Api endpoint that can be used to freely (within the allowed parameters) query the cve search database.

        The request sample payload displays a request body for a single query; multiple request can be combined within
        a comma separated list and send in a single request. In this case the responses will be send back in a list.
        For each request a separate list entry with the results.
        """
        headers = parse_headers(request.headers)
        database_connection = DatabaseHandler()

        output_format = ["json", "xml"]

        received_format = request.args.get("format", None)

        if received_format is None:
            format_output = format_output
        else:
            format_output = str(received_format)

        if format_output not in output_format:
            api.abort(
                400,
                "Specified output format is not possible; possible options are: {}!".format(
                    output_format
                ),
            )

        try:
            body = request.json
        except Exception:
            return "Could not parse request body", 400

        if isinstance(body, dict):
            result = database_connection.handle_api_json_query(
                JSONApiRequest(headers=headers, body=body)
            )
        elif isinstance(body, list):
            result = []
            for each in body:
                result.append(
                    database_connection.handle_api_json_query(
                        JSONApiRequest(headers=headers, body=each)
                    )
                )

        if isinstance(result, tuple):
            # error response; just return it
            return result
        else:
            if format_output == "json":
                return Response(
                    json.dumps(
                        result, indent=4, sort_keys=True, default=json_util.default
                    ),
                    mimetype="application/json",
                )
            if format_output == "xml":
                return Response(dicttoxml(result), mimetype="text/xml")



import re

from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, SubmitField
from wtforms.validators import InputRequired, Length, ValidationError


class LoginForm(FlaskForm):
    username = StringField(
        "username", validators=[InputRequired()], render_kw={"placeholder": "username"}
    )
    password = PasswordField(
        "password",
        validators=[InputRequired(), Length(min=5, max=24)],
        render_kw={"placeholder": "password"},
    )
    submit = SubmitField("Login")


def validate_username(form, field):
    user = users.query.filter_by(name=field.data).first()
    if user:
        raise ValidationError({"Username is already in use": []})


def validate_password(form, field):
    if field.data != form.confirm_password.data:
        raise ValidationError({"Passwords do not match": []})


def password_check(form, field):
    """
    Verify the strength of 'password'
    Returns a dict indicating the wrong criteria
    A password is considered strong if:
        8 characters length or more
        1 digit or more
        1 symbol or more
        1 uppercase letter or more
        1 lowercase letter or more
    """

    # calculating the length
    length_error = len(field.data) < 8

    # searching for digits
    digit_error = re.search(r"\d", field.data) is None

    # searching for uppercase
    uppercase_error = re.search(r"[A-Z]", field.data) is None

    # searching for lowercase
    lowercase_error = re.search(r"[a-z]", field.data) is None

    # searching for symbols
    symbol_error = (
        re.search(r"[ !#$%&'()*+,-./[\\\]^_`{|}~" + r'"]', field.data) is None
    )

    # overall result
    password_ok = not (
        length_error
        or digit_error
        or uppercase_error
        or lowercase_error
        or symbol_error
    )

    if password_ok is False:
        raise ValidationError(
            {
                "The provided password does not meet the following criteria:": [
                    "8 characters length or more",
                    "1 digit or more",
                    "1 symbol or more",
                    "1 uppercase letter or more",
                    "1 lowercase letter or more",
                ]
            }
        )


class RegistrationForm(FlaskForm):
    """
    Form for users to create new account
    """

    username = StringField(
        "username",
        validators=[validate_username],
        render_kw={"placeholder": "username"},
    )
    password = PasswordField(
        "password",
        validators=[validate_password, password_check],
        render_kw={"placeholder": "password"},
    )
    confirm_password = PasswordField(
        "Confirm password", render_kw={"placeholder": "confirm password"}
    )
    submit = SubmitField("Register")



#!/usr/bin/env python3
#
# Updater script of CVE/CPE database
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2019  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import logging
import os
import shlex
import subprocess
import sys
import time

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))


from lib.Config import Configuration

# pass configuration to CveXplore
Configuration.setCveXploreEnv()
from CveXplore import CveXplore

from lib.LogHandler import UpdateHandler
from lib.Sources_process import (
    CPERedisBrowser,
)
from lib.DatabaseLayer import (
    getSize,
    dropCollection,
    getTableNames,
    acquire_lock,
    release_lock,
    any_lock_active,
)

logging.setLoggerClass(UpdateHandler)

logger = logging.getLogger("DBUpdater")


def nbelement(collection=None):
    if collection is None or collection == "cve":
        collection = "cves"
    return getSize(collection)


def dropcollection(collection=None):
    if collection is None:
        return False
    if collection == "cve":
        collection = "cves"
    ret = dropCollection(collection)
    return ret


def _acquire_all_locks_or_none(sources):
    """
    Attempt to acquire locks for all sources.

    Returns:
        tuple(list_of_locked_names, failed_source_name):
            - list_of_locked_names: names of sources successfully locked
            - failed_source_name: name of the first source that could not be locked (None if all succeeded)
    """
    acquired = []
    for name in sources:
        if acquire_lock(name):
            acquired.append(name)
        else:
            # Release any locks we already acquired
            for s in acquired:
                release_lock(s)
            return None, name  # indicate which source caused failure
    return acquired, None


def main(args):
    cvex = CveXplore()

    # Repopulation

    if args.force:
        logger.info("==========================")
        logger.info("Repopulate")
        logger.info(time.strftime("%a %d %B %Y %H:%M", time.gmtime()))
        logger.info("==========================")

        repopulate_wait_max = 8
        repopulate_wait_interval = 300  # seconds
        for attempt in range(1, repopulate_wait_max + 1):
            if any_lock_active():
                logger.warning(
                    "Active updater locks detected. "
                    f"Waiting {repopulate_wait_interval} seconds for other db_updater processes to finish "
                    f"(Try {attempt}/{repopulate_wait_max})"
                )
                time.sleep(repopulate_wait_interval)
            else:
                break
        else:  # executes only if the loop did not break
            logger.error(
                "Active updater locks still detected after maximum wait. Aborting database repopulation."
            )
            sys.exit(1)

        # Lock all available CveXplore sources for the repopulation
        available_sources = []
        for source_available in cvex.database.sources:
            available_sources.append(source_available["name"])
        locked_sources, failed_source = _acquire_all_locks_or_none(available_sources)
        if not locked_sources:
            logger.error(f"Failed to lock {failed_source} for repopulation. Aborting.")
            sys.exit(1)
        try:
            to_drop = ["cpeother", "info", "schema"]
            for each in to_drop:
                logger.info(f"Dropping metadata: {each}")
                dropcollection(each)
            if args.loop and args.days == 0:
                logger.info(
                    "Drop collections (-f, --force) and running in loop (-l, --loop) used together; only dropping on the first iteration."
                )

            logger.info("Starting initial import...")
            cvex.database.initialize()
        finally:
            for name in locked_sources:
                release_lock(name)

    # Get sources from arguments or configuration and compare with sources provided by CveXplore

    update_sources = []
    ignored_sources = []
    if len(args.sources) > 0:
        logger.info(
            f'Using manually overridden sources instead of configured: {", ".join(args.sources)}'
        )
    for source_available in cvex.database.sources:
        if source_available["name"] in args.sources:
            update_sources.append(source_available["name"])
        elif len(args.sources) == 0 and Configuration.includesFeed(
            source_available["name"]
        ):
            update_sources.append(source_available["name"])
        else:
            ignored_sources.append(source_available["name"])
    if len(ignored_sources) > 0:
        logger.info(
            f'Ignored available CveXplore sources: {", ".join(ignored_sources)}'
        )
    if len(args.sources) > 0:
        unavailable_sources = [i for i in args.sources if i not in update_sources]
        if len(unavailable_sources) > 0:
            logger.warning(
                f'Sources unavailable in CveXplore: {", ".join(unavailable_sources)}'
            )
    if len(update_sources) == 0:
        logger.error(f"None of the sources available in CveXplore.")
        return 1

    # Update sources handled by CveXplore

    loop = True
    loop_count = 0

    while loop:
        if args.loop and args.days > 0:
            logger.warning(
                f"Loop (-l, --loop) not supported with manual days (-d, --days); only running once"
            )
        if not args.loop or args.days > 0:
            loop = False
        else:
            loop_count += 1

        logger.info("==========================")
        if args.cache:
            redis_info = " ; CPE redis cache enabled"
        else:
            redis_info = ""
        if args.days > 0:
            days_info = f" (manual interval of {str(args.days)} days)"
            loop_info = ""  # loop not supported with manual days
        else:
            days_info = ""
            if args.loop:
                loop_info = f" (loop #{loop_count})"
            else:
                loop_info = ""
        logger.info(
            f'Update [{", ".join(update_sources)}]{redis_info}{loop_info}{days_info}'
        )
        logger.info(time.strftime("%a %d %B %Y %H:%M", time.gmtime()))
        logger.info("==========================")

        locked_sources, failed_source = _acquire_all_locks_or_none(update_sources)
        if not locked_sources:
            if failed_source:
                if len(update_sources) == 1:
                    logger.warning(
                        f"Source {failed_source} is already locked; skipping CveXplore update"
                    )
                else:
                    logger.warning(
                        f"Could not acquire lock for source {failed_source}; skipping CveXplore batch update"
                    )
            else:
                logger.info(f"Nothing to update with CveXplore.")
        else:
            try:
                cvex.database.update(
                    update_source=update_sources, manual_days=args.days
                )
            finally:
                for name in locked_sources:
                    release_lock(name)

        # Update sources other than CveXplore

        sources = []
        if Configuration.includesFeed("cpeother"):
            sources.extend(
                [
                    {
                        "name": "cpeother",
                        "updater": "{} {}".format(
                            sys.executable,
                            os.path.join(runPath, "db_mgmt_cpe_other_dictionary.py"),
                        ),
                    }
                ]
            )
        if args.cache:
            sources.extend(
                [
                    {"name": "redis-cache-cpe", "updater": CPERedisBrowser},
                ]
            )

        for source in sources:
            if not acquire_lock(source["name"]):
                logger.warning(
                    f"Source {source['name']} is already locked by another update, skipping update"
                )
                continue
            try:
                # Drop collections only if this was the first iteration of a repopulation
                if (
                    args.force
                    and source["name"] != "redis-cache-cpe"
                    and loop_count < 2
                ):
                    logger.info("Repopulation; dropping collection: " + source["name"])
                    if dropcollection(collection=source["name"]):
                        logger.info(f"{source['name']} dropped")
                    else:
                        logger.info(f"{source['name']} did not exist")

                if (
                    not Configuration.includesFeed(source["name"])
                    and source["name"] != "redis-cache-cpe"
                ):
                    logger.info(f"Skipping non-configured source: {source['name']}")
                    continue

                if source["name"] == "cpeother":
                    if "cpeother" not in getTableNames():
                        continue
                if source["name"] != "redis-cache-cpe":
                    logger.info("Starting " + source["name"])
                    before = nbelement(collection=source["name"])

                    if isinstance(source["updater"], str):
                        subprocess.Popen((shlex.split(source["updater"]))).wait()
                    else:
                        up = source["updater"]()
                        up.update()

                    after = nbelement(collection=source["name"])
                    message = (
                        source["name"]
                        + " has "
                        + str(after)
                        + " elements ("
                        + str(after - before)
                        + " update)"
                    )
                    logger.info(message)
                elif args.cache is True and source["name"] == "redis-cache-cpe":
                    logger.info("Starting " + source["name"])
                    up = source["updater"]()
                    up.update()
                    logger.info(source["name"] + " updated")
            finally:
                release_lock(source["name"])

        if args.index:
            logger.warning(
                "Fulltext indexing has been removed from db_updater.py. "
                "Use db_fulltext.py to manage fulltext index."
            )

        if args.loop and args.days == 0:
            logger.info("Sleeping 1 hour...")
            time.sleep(3600)


if __name__ == "__main__":
    argParser = argparse.ArgumentParser(description="Database updater for cve-search")
    argParser.add_argument(
        "-s",
        "--sources",
        nargs="*",
        metavar="SOURCE",
        help="Sources to be updated if available in CveXplore. Defaults to all sources available & configured.",
        default=[],
    )
    argParser.add_argument(
        "-i",
        "--index",
        action="store_true",
        help=(
            "Dummy option for backwards compatibility. ",
            "Use sbin/db_fulltext.py for indexing CVE entries in the fulltext indexer",
        ),
        default=False,
    )
    argParser.add_argument(
        "-l",
        "--loop",
        action="store_true",
        help="Running at regular interval; waits 1 hour (disabled with -d, --days)",
        default=False,
    )
    argParser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Drop collections and force initial import (only on first iteration with -l, --loop)",
        default=False,
    )
    argParser.add_argument(
        "-d",
        "--days",
        type=int,
        choices=range(1, 121),
        metavar="1..120",
        help="Set update interval (1-120 days) manually for NVD API (CPE, CVE)",
        default=0,  # not manually set; updates CPE & CVE since last update
    )
    argParser.add_argument(
        "-c",
        "--cache",
        action="store_true",
        help="Enable CPE redis cache",
        default=False,
    )
    args = argParser.parse_args()

    main(args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Software is free software released under the "Modified BSD license"
#
# Copyright (c) 2014       psychedelys
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com
# Copyright (c) 2015-2019  Alexandre Dulaunoy - a@foo.be

import argparse
import json
import os
import re
import sys
import urllib.parse

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

import lib.DatabaseLayer as db

runPath = os.path.dirname(os.path.realpath(__file__))

vOutput = ""

argParser = argparse.ArgumentParser(description="Search for CPE with a pattern")
argParser.add_argument("-s", type=str, required=True, help="search in cpe list")
argParser.add_argument(
    "-o",
    type=str,
    default="expanded",
    help="O = output format [expanded, compact, json, csv] (default: expanded)",
)
argParser.add_argument(
    "-f",
    action="store_true",
    help="Enlarge the CPE search to all CPE indexed. Need the cpeother activated.",
    default=False,
)

args = argParser.parse_args()
cpeSearch = args.s
vOutput = args.o


def search(cpe):
    res = db.getCPEMatching(re.compile(cpe, re.IGNORECASE), args.f)

    if vOutput == "compact":
        for item in res:
            print("{}".format(item["id"]))
    elif vOutput == "expanded":
        for item in res:
            print("{} {}".format(item["id"], item["title"]))
    elif vOutput == "csv":
        for item in res:
            if "references" in item:
                ref = ",".join(item["references"])
                print("{},{},{}".format(item["id"], item["title"], ref))
            else:
                print("{},{}".format(item["id"], item["title"]))
    elif vOutput == "json":
        o = []
        for item in res:
            x = {}
            x["id"] = item["id"]
            x["title"] = item["title"]
            if "references" in item:
                x["references"] = item["references"]
            o.append(x)
        print(json.dumps(o, sort_keys=True, indent=4))


# replace special characters in cpeSearch with encoded version.
cpeSearch = urllib.parse.quote(cpeSearch)

search(cpeSearch)



import re
from collections import defaultdict

from flask import render_template, request, make_response, json, jsonify, url_for
from flask_breadcrumbs import register_breadcrumb, default_breadcrumb_root

from lib.CVEs import CveHandler
from . import home
from .utils import (
    defaultFilters,
    config_args,
    generate_minimal_query,
    validateFilter,
    SanitizeUserInput,
)
from ..helpers.server_side_datatables import ServerSideDataTable
from ..run import app

default_breadcrumb_root(home, ".")


@home.route("/", methods=["GET"])
@register_breadcrumb(home, ".", "Home")
def index():
    return render_template("index.html", **config_args)


def view_vendor_name(*args, **kwargs):
    try:
        return [
            {"text": "Vendor-List", "url": "{}browse".format(url_for("home.index"))},
            {
                "text": "{}".format(request.view_args["vendor"]),
                "url": "{}browse/{}".format(
                    url_for("home.index"), request.view_args["vendor"]
                ),
            },
        ]
    except KeyError:
        return [{"text": "Vendor-List", "url": "browse"}]


@home.route("/browse")
@home.route("/browse/<vendor>")
@register_breadcrumb(home, ".browse", "", dynamic_list_constructor=view_vendor_name)
def browse_vendor(vendor=None):
    if vendor is None:
        return render_template("browse_vendor.html", **config_args)

    else:
        data = {
            "vendor": vendor,
            "product": [
                x
                for x in app.dbh.connection.store_cves.distinct(
                    "products", {"vendors": vendor}
                )
                if x is not None
            ],
        }

        return render_template(
            "browse.html", product=data["product"], vendor=data["vendor"]
        )


@home.route("/browse/fetch_data", methods=["POST"])
def browse_fetch_data():
    retdata = {
        "data": [
            {"vendors": x}
            for x in app.dbh.connection.store_cves.distinct("vendors")
            if x is not None
        ]
    }

    return retdata


def view_cve_id_name(*args, **kwargs):
    return [
        {
            "text": request.view_args["cve_id"],
            "url": "{}cve/{}".format(
                url_for("home.index"), request.view_args["cve_id"]
            ),
        }
    ]


@home.route("/cve/<cve_id>")
@register_breadcrumb(home, ".cve", "", dynamic_list_constructor=view_cve_id_name)
def cve(cve_id):
    cvesp = CveHandler(
        rankinglookup=True,
        namelookup=True,
        via4lookup=True,
        capeclookup=True,
        subscorelookup=True,
    )
    cve = cvesp.getcve(cveid=cve_id.upper())
    if cve is None:
        return (
            render_template(
                "error.html",
                status={"except": "cve-not-found", "info": {"cve": cve_id}},
            ),
            404,
        )
    else:
        return render_template("cve.html", cve=cve)


def view_cwe_id_name(*args, **kwargs):
    try:
        return [
            {"text": "CWE-List", "url": "{}cwe".format(url_for("home.index"))},
            {
                "text": "CWE-{}".format(request.view_args["cwe_id"]),
                "url": "{}cwe/{}".format(
                    url_for("home.index"), request.view_args["cwe_id"]
                ),
            },
        ]
    except KeyError:
        return [{"text": "CWE-List", "url": "{}cwe".format(url_for("home.index"))}]


@home.route("/cwe")
@home.route("/cwe/<cwe_id>")
@register_breadcrumb(home, ".cwe", "", dynamic_list_constructor=view_cwe_id_name)
def get_cwe(cwe_id=None):
    from lib.DatabaseLayer import (
        getCWEs,
        getCAPECFor,
    )

    data = getCWEs()
    if cwe_id is None:
        cwes = [x for x in data if x["weaknessabs"].lower() == "class"]
        return render_template("cwe.html", cwes=cwes, capec=None)
    else:
        cwes = {x["id"]: x["name"] for x in data}
        return render_template(
            "cwe.html", cwes=cwes, cwe=cwe_id, capec=getCAPECFor(cwe_id), minimal=True
        )


def view_capec_id_name(*args, **kwargs):
    return [
        {
            "text": "CAPEC-{}".format(request.view_args["capec_id"]),
            "url": "{}capec/{}".format(
                url_for("home.index"), request.view_args["capec_id"]
            ),
        }
    ]


@home.route("/capec/<capec_id>")
@register_breadcrumb(home, ".capec", "", dynamic_list_constructor=view_capec_id_name)
def capec(capec_id):
    from lib.DatabaseLayer import (
        getCWEs,
        getCAPEC,
    )

    data = getCWEs()
    cwes = {x["id"]: x["name"] for x in data}

    req_capec = getCAPEC(capec_id)
    if req_capec is None:
        return render_template(
            "error.html",
            status={"except": "capec-not-found", "info": {"capec": capec_id}},
        )

    rel_capecs = defaultdict(dict)

    if len(req_capec["related_capecs"]) != 0:
        for each in req_capec["related_capecs"]:
            rel_capecs[each] = getCAPEC(each)["summary"]

    return render_template(
        "capec.html", cwes=cwes, capecs=dict(rel_capecs), capec=req_capec
    )


def view_product_name(*args, **kwargs):
    vendor = request.view_args["vendor"]

    return [
        {
            "text": "{}".format(request.view_args["product"]),
            "url": "{}search/{}/{}".format(
                url_for("home.index"), vendor, request.view_args["product"]
            ),
        }
    ]


@home.route("/search/<vendor>/<path:product>")
@register_breadcrumb(
    home, ".browse.vendor", "", dynamic_list_constructor=view_product_name
)
def search(vendor=None, product=None):
    return render_template(
        "search.html", freetextsearch="", vendor=vendor, product=product, **config_args
    )


def view_freetext_search(*args, **kwargs):
    try:
        return [
            {
                "text": "Search: {}".format(request.view_args["freetextsearch"]),
                "url": "{}search/{}".format(
                    url_for("home.index"), request.view_args["freetextsearch"]
                ),
            }
        ]
    except KeyError:
        return [{"text": "Search", "url": ""}]


@home.route("/search/<freetextsearch>")
@register_breadcrumb(home, ".search", "", dynamic_list_constructor=view_freetext_search)
def freetext_search(freetextsearch=None):
    return render_template(
        "search.html",
        freetextsearch=freetextsearch,
        vendor="",
        product="",
        **config_args
    )


@home.route("/fetch_search_data", methods=["POST"])
def fetch_freetext_search():
    from lib.DatabaseLayer import (
        cvesForCPE,
        getSearchResults,
    )

    search = ""
    if request.values.get("search"):
        search = request.values.get("search")
    vendor = ""
    if request.values.get("vendor"):
        vendor = request.values.get("vendor")
    product = ""
    if request.values.get("product"):
        product = request.values.get("product")

    if search != "":
        result = getSearchResults(search)
        cve = {"data": result["data"], "total": len(result["data"])}
    elif vendor != "" and product != "":
        search = (vendor, product)
        result = cvesForCPE(search, strict_vendor_product=True)
        cve = {"data": result["results"], "total": len(result["results"])}
    else:
        return make_response(jsonify(False), 400)

    # errors = result["errors"] if "errors" in result else []
    return make_response(jsonify(cve), 200)


def view_linked(*args, **kwargs):
    key = request.view_args["key"]

    return [
        {
            "text": "{}".format(request.view_args["value"]),
            "url": "{}link/{}/{}".format(
                url_for("home.index"), key, request.view_args["value"]
            ),
        }
    ]


@home.route("/link/<key>/<value>")
@register_breadcrumb(home, ".linked", "", dynamic_list_constructor=view_linked)
def link(key=None, value=None):
    from lib.DatabaseLayer import (
        via4Linked,
    )

    regex = re.compile(re.escape(value), re.I)
    cve = via4Linked(key, regex)
    cvssList = [float(x["cvss"]) for x in cve["results"] if x.get("cvss")]
    if cvssList:
        stats = {
            "maxCVSS": max(cvssList),
            "minCVSS": min(cvssList),
            "count": int(cve["total"]),
        }
    else:
        stats = {"maxCVSS": 0, "minCVSS": 0, "count": int(cve["total"])}
    return render_template(
        "linked.html",
        via4map=key.split(".")[0],
        field=".".join(key.split(".")[1:]),
        value=value,
        cve=cve,
        stats=stats,
    )


@home.route("/set_filter", methods=["POST"])
def set_filter():
    try:
        filter_params = dict(request.json)
        if validateFilter(filter_params):
            resp = make_response(jsonify(True), 200)
            resp.set_cookie("cve_filter", json.dumps(filter_params))
        else:
            resp = make_response(jsonify(False), 400)
    except TypeError:
        resp = make_response(jsonify(False), 400)
    return resp


@home.route("/reset_filter")
def reset_filter():
    resp = make_response(jsonify(defaultFilters), 200)
    resp.set_cookie("cve_filter", "", expires=0)
    return resp


@home.route("/filter_active")
def filter_active():
    try:
        filter_params = dict(json.loads(request.cookies.get("cve_filter")))
        if not validateFilter(filter_params):
            filter_params = defaultFilters
    except TypeError:
        filter_params = defaultFilters

    if filter_params == defaultFilters:
        resp = make_response(jsonify(False), 200)
        resp.set_cookie("cve_filter", "", expires=0)
    else:
        resp = make_response(jsonify(True), 200)
    return resp


@home.route("/get_filter")
def get_filter():
    try:
        filter_params = SanitizeUserInput(
            dict(json.loads(request.cookies.get("cve_filter")))
        )
        if not validateFilter(filter_params):
            filter_params = defaultFilters
    except TypeError:
        filter_params = defaultFilters
    return filter_params


@home.route("/fetch_cve_data", methods=["POST"])
def fetch_cvedata():
    # Enforce endpoint constraints: this API only serves CVE data
    retrieve = request.form.get("retrieve")
    if retrieve != "cves":
        return make_response(
            jsonify(error="invalid retrieve parameter"),
            400,
        )

    try:
        filter_params = dict(json.loads(request.cookies.get("cve_filter")))
        if not validateFilter(filter_params):
            filter_params = defaultFilters
    except TypeError:
        filter_params = defaultFilters

    try:
        if filter_params == defaultFilters:
            ssd = ServerSideDataTable(request=request, backend=app.dbh.connection)
        else:
            query_filters = generate_minimal_query(filter_params)
            if len(query_filters) != 0:
                ssd = ServerSideDataTable(
                    request=request,
                    backend=app.dbh.connection,
                    additional_filters=query_filters,
                )
            else:
                ssd = ServerSideDataTable(request=request, backend=app.dbh.connection)
    except ValueError as err:
        return make_response(
            jsonify(error=str(err)),
            400,
        )

    return_data = ssd.output_result()

    return return_data



from lib.Authentication import AuthenticationHandler


class FlaskAuthHandler:
    def __init__(self, app=None, **kwargs):
        self.kwargs = kwargs
        self.auth = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app, **kwargs):
        self.kwargs.update(kwargs)
        app.auth_handler = self

    def __getattr__(self, name):
        if self.auth is None:
            self.auth = AuthenticationHandler()
        return getattr(self.auth, name)

    def __repr__(self):
        return "<< FlaskAuthHandler >>"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Dump last CVE entries in RSS 1,RSS 2 or Atom format.
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import datetime
import html
import os
import sys
import time

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.CVEs import CveHandler

argParser = argparse.ArgumentParser(
    description="Dump last CVE entries in RSS/Atom format or in HTML tables"
)
argParser.add_argument(
    "-f", type=str, help="Output format (rss1,rss2,atom,html)", default="rss1"
)
argParser.add_argument("-l", type=int, help="Last n items (default:10)", default=10)
argParser.add_argument(
    "-n", action="store_false", help="Disable lookup CPE name (default is True)"
)
argParser.add_argument(
    "-r",
    action="store_true",
    help="Enable CVE ranking (default is False) and only print entries with ranking",
)
argParser.add_argument(
    "-c", default=False, action="store_true", help="Display CAPEC values"
)

args = argParser.parse_args()

if args.l:
    last_items = args.l
else:
    last_items = 10

ref = "https://github.com/cve-search/cve-search/"
cvelist = CveHandler(rankinglookup=args.r, namelookup=args.n, capeclookup=args.c)

if not (args.f == "html"):
    from feedformatter import Feed

    feed = Feed()

    feed.feed["title"] = (
        "cve-search Last "
        + str(last_items)
        + " CVE entries generated on "
        + str(datetime.datetime.now())
    )
    feed.feed["link"] = "https://github.com/cve-search/cve-search/"
    feed.feed["author"] = (
        "Generated with cve-search available at https://github.com/cve-search/cve-search/"
    )
    feed.feed["description"] = ""
else:
    print("<html><head>")
    print(
        "<style>.cve table { border-collapse: collapse; text-align: left; width: 100%; } .cve {font: normal 12px/150% "
        "Geneva, Arial, Helvetica, sans-serif; background: #fff; overflow: hidden; border: 1px solid #006699; "
        "-webkit-border-radius: 3px; -moz-border-radius: 3px; border-radius: 3px; }.cve table td, .cve table th { "
        "padding: 3px 10px; }.cve table tbody td { color: #00496B; border-left: 1px solid #E1EEF4;font-size: "
        "12px;font-weight: normal; }.cve table tbody .alt td { background: #E1EEF4; color: #00496B; }.cve table tbody "
        "td:first-child { border-left: none; }.cve table tbody tr:last-child td { border-bottom: none; }.cve table "
        "tfoot td div { border-top: 1px solid #006699;background: #E1EEF4;} .cve table tfoot td { padding: 0; "
        "font-size: 12px } .cve table tfoot td div{ padding: 0px; }</style> "
    )
    print("<title>Last " + str(args.l) + " CVE entries</title>")
    print("</head><body>")
for x in cvelist.get(limit=last_items):
    if not (args.f == "html"):
        item = {
            "title": str(x["id"]) + " " + x["summary"][:90] + "...",
            "description": x["summary"],
        }
        if args.r and x.get("ranking"):
            item["description"] = item["description"] + " Ranking:" + str(x["ranking"])
        item["pubDate"] = time.localtime()
        item["guid"] = x["id"]
        if x["references"]:
            item["link"] = str(x["references"][0])
        else:
            item["link"] = "https://nvd.nist.gov/vuln/detail/" + x["id"]
        feed.items.append(item)
    else:
        print('<div class="cve"><table><tbody>')
        print('<tr class="alt">')
        print(
            "<td>"
            + str(x.get("id", ""))
            + " - "
            + html.escape(x.get("summary", "")[:90])
            + "...</td>"
        )
        print("</tr>")
        print(
            "<tr><td>CVSS: "
            + str(x.get("cvss", "N/A"))
            + " published: "
            + str(x.get("published", "N/A"))
            + "</td></tr>"
        )
        print("<tr>")
        print("<td> Summary: " + html.escape(x.get("summary", "")) + "</td>")
        print("</tr>")

        vc = x.get("vulnerable_configuration", [])
        if vc:
            print("<tr><td>Vulnerable configuration:</td></tr>")
            print("<tr><td><ul>")
            for v in vc:
                if isinstance(v, dict):
                    label = v.get("title") or v.get("id", "")
                    sys.stdout.write(f"<li>{html.escape(label)}</li>")
                else:
                    sys.stdout.write(f"<li>{html.escape(cvelist.getcpe(v))}</li>")
            print("</ul></td></tr>")

        vp = x.get("vulnerable_product", [])
        if vp:
            print("<tr><td>Vulnerable products:</td></tr>")
            print("<tr><td><ul>")
            for p in vp:
                sys.stdout.write(f"<li>{html.escape(cvelist.getcpe(p))}</li>")
            print("</ul></td></tr>")

        if x.get("ranking"):
            print(f"<tr><td>Ranking: {html.escape(str(x['ranking']))}</td></tr>")

        refs = x.get("references", [])
        if refs:
            print("<tr><td>References:</td></tr>")
            print("<tr><td><ul>")
            for r in refs:
                if r:  # ignore empty strings/nulls
                    safe_r = html.escape(str(r))
                    sys.stdout.write(f'<li><a href="{safe_r}">{safe_r}</a></li>')
            print("</ul></td></tr>")

        print("</tbody></table></div><br/>")
if args.f == "rss1":
    print(feed.format_rss1_string())
elif args.f == "atom":
    print(feed.format_atom_string())
elif args.f == "html":
    print(
        'Generated with <a href="https://github.com/cve-search/cve-search/">cve-search</a> at '
        + str(datetime.datetime.today())
        + "."
    )
    print("</body></html>")
else:
    print(feed.format_rss2_string())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Config reader to read the configuration file
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2013-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import bz2
import configparser
import gzip
import json
import os
import re
import urllib.parse
import urllib.request as req
import zipfile
from io import BytesIO

import pymongo
import redis

runPath = os.path.dirname(os.path.realpath(__file__))


class Configuration:
    ConfigParser = configparser.ConfigParser()
    ConfigParser.read(
        [
            os.path.join(runPath, "../etc/configuration.ini"),
            os.path.join(runPath, "../etc/sources.ini"),
        ]
    )
    default = {
        # [Redis]
        "redisHost": "localhost",
        "redisPort": 6379,
        "redisPass": None,
        "redisVendorDB": 10,
        "redisNotificationsDB": 11,
        # [Database] (MongoDB)
        "mongoHost": "127.0.0.1",
        "mongoPort": 27017,
        "mongoDB": "cvedb",
        "mongoUsername": "",
        "mongoPassword": "",
        "mongoSrv": False,
        "mongoAuth": "admin",
        "mongoLockMaxDurationSec": 10800,
        "DatabasePluginName": "mongodb",
        # [Download]
        "DownloadMaxWorkers": 10,
        # [dbmgt]
        "Tmpdir": "./tmp",
        # [FulltextIndex]
        "Indexdir": "./indexdir",
        # [Webserver]
        "flaskHost": "127.0.0.1",
        "flaskPort": 5000,
        "flaskDebug": True,
        "flaskSSLDebug": False,
        "pageLength": 50,
        "loginRequired": False,
        "auth_load": "./etc/auth.txt",
        "oidc": False,
        "client_id": "xx",
        "client_secret": "xx",
        "idp_discovery_url": "xx",
        "ssl_verify": False,
        "ssl": False,
        "sslCertificate": "./ssl/cve-search.crt",
        "sslKey": "./ssl/cve-search.crt",
        "WebInterface": "Full",  # defaults to Full; choices are 'Full' or 'Minimal'
        "MountPath": "/MOUNT",  # must never end with a backslash...
        # [API]
        "CVEMaxLimit": 1000,
        "CORS": False,
        "CORS_Allow_Origin": "*",
        # [Logging]
        "logging": True,
        "logfile": "./log/cve-search.log",
        "updatelogfile": "./log/update_populate.log",
        "maxLogSize": "100MB",
        "backlog": 5,
        "redisFallbackWarnings": True,
        # [Proxy]
        "http_proxy": "",
        # [CVE]
        "CVEStartYear": 2002,
    }

    sources = {
        "cwe": "https://cwe.mitre.org/data/xml/cwec_latest.xml.zip",
        "capec": "https://capec.mitre.org/data/xml/capec_latest.xml",
        "via4": "https://www.cve-search.org/feeds/via4.json",
        "epss": "https://epss.empiricalsecurity.com/epss_scores-current.csv.gz",
    }

    included = {
        "cpe": True,
        "cve": True,
        "cwe": True,
        "capec": True,
        "via4": True,
        "epss": True,
    }

    @classmethod
    def setCveXploreEnv(cls):
        """Wrapper for passing all the environment variables to CveXplore"""
        cls.setMongoDBEnv()
        cls.setProxyEnv()
        cls.setSourcesEnv()

    @classmethod
    def reloadConfiguration(cls):
        cls.ConfigParser.clear()
        return cls.ConfigParser.read(
            [
                os.path.join(runPath, "../etc/configuration.ini"),
                os.path.join(runPath, "../etc/sources.ini"),
            ]
        )

    @classmethod
    def readSetting(cls, section, item, default):
        result = default
        try:
            if type(default) == bool:
                result = cls.ConfigParser.getboolean(section, item)
            elif type(default) == int:
                result = cls.ConfigParser.getint(section, item)
            else:
                result = cls.ConfigParser.get(section, item)
        except:
            pass
        return result

    @classmethod
    def getWebInterface(cls):
        return cls.readSetting("Webserver", "WebInterface", cls.default["WebInterface"])

    @classmethod
    def getMountPath(cls):
        return cls.readSetting("Webserver", "MountPath", cls.default["MountPath"])

    # Mongo

    @classmethod
    def getMongoHost(cls):
        return cls.readSetting("Database", "Host", cls.default["mongoHost"])

    @classmethod
    def getMongoPort(cls):
        return cls.readSetting("Database", "Port", cls.default["mongoPort"])

    @classmethod
    def getMongoDB(cls):
        """Return the database name, optionally overridden for tests."""
        db_name = cls.readSetting("Database", "DB", cls.default["mongoDB"])
        if os.getenv("USE_TEST_DB") == "1":
            db_name = f"{db_name}_test"
        return db_name

    @classmethod
    def getMongoUsername(cls):
        return cls.readSetting("Database", "Username", cls.default["mongoUsername"])

    @classmethod
    def getMongoPassword(cls):
        return cls.readSetting("Database", "Password", cls.default["mongoPassword"])

    @classmethod
    def getMongoSrvBool(cls) -> bool:
        """Returns DnsSrvRecord as a boolean, as configparser.ConfigParser() returns strings"""
        val = cls.readSetting("Database", "DnsSrvRecord", cls.default["mongoSrv"])
        return str(val).lower() in ("true", "1", "yes")

    @classmethod
    def getMongoAuthDB(cls):
        return cls.readSetting("Database", "AuthDB", cls.default["mongoAuth"])

    @classmethod
    def getMongoLockMaxDurationSec(cls) -> int:
        value = cls.readSetting(
            "Database",
            "LockMaxDurationSec",
            cls.default["mongoLockMaxDurationSec"],
        )
        try:
            return int(value)
        except (TypeError, ValueError):
            return cls.default["mongoLockMaxDurationSec"]

    @classmethod
    def getMongoPluginName(cls):
        return cls.readSetting(
            "Database", "PluginName", cls.default["DatabasePluginName"]
        )

    @classmethod
    def getMongoConnection(cls):
        # jdt_NOTE: now correctly catches exceptions due to changes in pymongo 2.9 or later
        # jdt_NOTE: https://api.mongodb.com/python/current/migrate-to-pymongo3.html#mongoclient-connects-asynchronously
        connect = pymongo.MongoClient(cls.getMongoUri(), connect=False)
        return connect[cls.getMongoDB()]

    @classmethod
    def getMongoUri(cls):
        username = urllib.parse.quote(cls.getMongoUsername())
        password = urllib.parse.quote(cls.getMongoPassword())
        host = cls.getMongoHost()
        port = cls.getMongoPort()
        db = cls.getMongoDB()
        auth_db = cls.getMongoAuthDB()

        if cls.getMongoSrvBool():
            scheme = "mongodb+srv://"
            address = f"{host}"  # no port for SRV
            cluster_params = "retryWrites=true&w=majority"
        else:
            scheme = "mongodb://"
            address = f"{host}:{port}"
            cluster_params = ""

        authentication = f"{username}:{password}@" if username and password else ""
        auth_params = f"authSource={auth_db}" if username and password else ""

        # join params safely and cleanly
        query_parts = [p for p in (auth_params, cluster_params) if p]
        query = "&".join(query_parts)
        query_str = f"?{query}" if query else ""

        return f"{scheme}{authentication}{address}/{db}{query_str}"

    @classmethod
    def setMongoDBEnv(cls):
        """Sets MongoDB settings as environment variables for CveXplore"""
        os.environ["DATASOURCE_HOST"] = cls.getMongoHost()
        os.environ["DATASOURCE_PORT"] = str(cls.getMongoPort())
        os.environ["DATASOURCE_DBNAME"] = cls.getMongoDB()

        # Pass DnsSrvRecord configuration
        if cls.getMongoSrvBool():
            os.environ["DATASOURCE_DBAPI"] = "srv"

        # Both username & password should be set to make any sense
        if cls.getMongoUsername() == "" or cls.getMongoPassword() == "":
            # Unset environment variables to make CveXplore default to None
            os.environ.pop("DATASOURCE_USER", None)
            os.environ.pop("DATASOURCE_PASSWORD", None)
        else:
            os.environ["DATASOURCE_USER"] = cls.getMongoUsername()
            os.environ["DATASOURCE_PASSWORD"] = cls.getMongoPassword()

    @classmethod
    def toPath(cls, path):
        return path if os.path.isabs(path) else os.path.join(runPath, "..", path)

    # Redis
    @classmethod
    def getRedisHost(cls):
        return cls.readSetting("Redis", "Host", cls.default["redisHost"])

    @classmethod
    def getRedisPort(cls):
        return cls.readSetting("Redis", "Port", cls.default["redisPort"])

    @classmethod
    def getRedisVendorConnection(cls):
        redisHost = cls.getRedisHost()
        redisPort = cls.getRedisPort()
        redisDB = cls.readSetting("Redis", "VendorsDB", cls.default["redisVendorDB"])
        redisPass = cls.readSetting("Redis", "Password", cls.default["redisPass"])
        return redis.StrictRedis(
            host=redisHost,
            port=redisPort,
            db=redisDB,
            password=redisPass,
            charset="utf-8",
            decode_responses=True,
        )

    @classmethod
    def getRedisNotificationsConnection(cls):
        redisHost = cls.getRedisHost()
        redisPort = cls.getRedisPort()
        redisDB = cls.readSetting(
            "Redis", "NotificationsDB", cls.default["redisNotificationsDB"]
        )
        redisPass = cls.readSetting("Redis", "Password", cls.default["redisPass"])
        return redis.StrictRedis(
            host=redisHost,
            port=redisPort,
            db=redisDB,
            password=redisPass,
            charset="utf-8",
            decode_responses=True,
        )

    # Flask
    @classmethod
    def getFlaskHost(cls):
        return cls.readSetting("Webserver", "Host", cls.default["flaskHost"])

    @classmethod
    def getFlaskPort(cls):
        return cls.readSetting("Webserver", "Port", cls.default["flaskPort"])

    @classmethod
    def getFlaskDebug(cls):
        return cls.readSetting("Webserver", "Debug", cls.default["flaskDebug"])

    @classmethod
    def getFlaskSSLDebug(cls):
        return cls.readSetting("Webserver", "SSLDebug", cls.default["flaskSSLDebug"])

    # Webserver
    @classmethod
    def getPageLength(cls):
        return cls.readSetting("Webserver", "PageLength", cls.default["pageLength"])

    # REST API
    @classmethod
    def getCVEMaxLimit(cls):
        return cls.readSetting("API", "CVEMaxLimit", cls.default["CVEMaxLimit"])

    @classmethod
    def getCORS(cls):
        return cls.readSetting("API", "CORS", cls.default["CORS"])

    @classmethod
    def getCORSAllowOrigin(cls):
        return cls.readSetting(
            "API", "CORS_Allow_Origin", cls.default["CORS_Allow_Origin"]
        )

    # Authentication
    @classmethod
    def loginRequired(cls):
        return cls.readSetting(
            "Webserver", "LoginRequired", cls.default["loginRequired"]
        )

    @classmethod
    def getAuthLoadSettings(cls):
        return cls.toPath(
            cls.readSetting("Webserver", "authSettings", cls.default["auth_load"])
        )

    @classmethod
    def useOIDC(cls):
        return cls.readSetting("Webserver", "OIDC", cls.default["oidc"])

    @classmethod
    def getClientID(cls):
        return cls.readSetting("Webserver", "CLIENT_ID", cls.default["client_id"])

    @classmethod
    def getClientSecret(cls):
        return cls.readSetting(
            "Webserver", "CLIENT_SECRET", cls.default["client_secret"]
        )

    @classmethod
    def getIDPDiscoveryUrl(cls):
        return cls.readSetting(
            "Webserver", "IDP_DISCOVERY_URL", cls.default["idp_discovery_url"]
        )

    # SSL
    @classmethod
    def useSSLVerify(cls):
        return cls.readSetting("Webserver", "SSL_VERIFY", cls.default["ssl_verify"])

    @classmethod
    def getSSLCert(cls):
        return cls.toPath(
            cls.readSetting("Webserver", "Certificate", cls.default["sslCertificate"])
        )

    @classmethod
    def getSSLKey(cls):
        return cls.toPath(cls.readSetting("Webserver", "Key", cls.default["sslKey"]))

    # Logging
    @classmethod
    def getLogfile(cls):
        return cls.toPath(cls.readSetting("Logging", "Logfile", cls.default["logfile"]))

    @classmethod
    def getUpdateLogFile(cls):
        return cls.toPath(
            cls.readSetting("Logging", "Updatelogfile", cls.default["updatelogfile"])
        )

    @classmethod
    def getLogging(cls):
        return cls.readSetting("Logging", "Logging", cls.default["logging"])

    @classmethod
    def getMaxLogSize(cls):
        size = cls.readSetting("Logging", "MaxSize", cls.default["maxLogSize"])
        split = re.findall("\\d+|\\D+", size)
        multipliers = {"KB": 1024, "MB": 1024 * 1024, "GB": 1024 * 1024 * 1024}
        if len(split) == 2:
            base = int(split[0])
            unit = split[1].strip().upper()
            return base * multipliers.get(unit, 1024 * 1024)
        # if size is not a correctly defined set it to 100MB
        else:
            return 100 * 1024 * 1024

    @classmethod
    def getBacklog(cls):
        return cls.readSetting("Logging", "Backlog", cls.default["backlog"])

    @classmethod
    def getRedisFallbackWarnings(cls):
        value = cls.readSetting(
            "Logging", "RedisFallbackWarnings", cls.default["redisFallbackWarnings"]
        )
        if isinstance(value, bool):
            return value
        if isinstance(value, str):
            return value.strip().lower() in ("true", "1", "yes", "on")
        return bool(value)

    # Download Max Workers
    @classmethod
    def getDownloadMaxWorkers(cls):
        maxWorkers = cls.readSetting(
            "Download", "MaxWorkers", cls.default["DownloadMaxWorkers"]
        )
        if type(maxWorkers) == int:
            if maxWorkers > 0:
                return maxWorkers
            else:
                return 1
        else:
            return cls.default["DownloadMaxWorkers"]

    # Indexing
    @classmethod
    def getIndexdir(cls):
        return cls.toPath(
            cls.readSetting("FulltextIndex", "Indexdir", cls.default["Indexdir"])
        )

    # Http Proxy
    @classmethod
    def getProxy(cls):
        return cls.readSetting("Proxy", "http", cls.default["http_proxy"])

    @classmethod
    def setProxyEnv(cls):
        """Sets proxy settings as environment variables for CveXplore"""
        if cls.getProxy() == "":
            # Remove environment variables
            os.environ.pop("HTTP_PROXY_DICT", None)
            os.environ.pop("HTTP_PROXY_STRING", None)
        else:
            # Set environment variables
            proxyDict = {
                "http": f"http://{cls.getProxy()}",
                "https": f"http://{cls.getProxy()}",
            }
            os.environ["HTTP_PROXY_DICT"] = json.dumps(proxyDict)
            os.environ["HTTP_PROXY_STRING"] = f"http://{cls.getProxy()}"

    @classmethod
    def getFile(cls, getfile, unpack=True):
        if cls.getProxy():
            proxy = req.ProxyHandler({"http": cls.getProxy(), "https": cls.getProxy()})
            auth = req.HTTPBasicAuthHandler()
            opener = req.build_opener(proxy, auth, req.HTTPHandler)
            req.install_opener(opener)

        response = req.urlopen(getfile)
        data = response
        # TODO: if data == text/plain; charset=utf-8, read and decode
        if unpack:
            if "gzip" in response.info().get("Content-Type"):
                buf = BytesIO(response.read())
                data = gzip.GzipFile(fileobj=buf)
            elif "bzip2" in response.info().get("Content-Type"):
                data = BytesIO(bz2.decompress(response.read()))
            elif "zip" in response.info().get("Content-Type"):
                fzip = zipfile.ZipFile(BytesIO(response.read()), "r")
                if len(fzip.namelist()) > 0:
                    data = BytesIO(fzip.read(fzip.namelist()[0]))
        return (data, response)

    # Sources
    @classmethod
    def getFeedURL(cls, source):
        return cls.readSetting("Sources", source, cls.sources.get(source, ""))

    @classmethod
    def includesFeed(cls, feed):
        return cls.readSetting("EnabledFeeds", feed, cls.included.get(feed, False))

    @classmethod
    def setSourcesEnv(cls):
        """Sets sources dictionary as an environment variable for CveXplore"""
        sources = {}
        for source in cls.sources:
            sources.update({source: cls.getFeedURL(source)})
        os.environ["SOURCES"] = json.dumps(sources)


class ConfigReader:
    def __init__(self, file):
        self.ConfigParser = configparser.ConfigParser()
        self.ConfigParser.read(file)

    def read(self, section, item, default):
        result = default
        try:
            if type(default) == bool:
                result = self.ConfigParser.getboolean(section, item)
            elif type(default) == int:
                result = self.ConfigParser.getint(section, item)
            else:
                result = self.ConfigParser.get(section, item)
        except:
            pass
        return result



from flask_restx import Namespace, Resource, fields

from web.restapi.cpe_convert import message

api = Namespace("cwe", description="Endpoints for requesting cwe information", path="/")

cwe = api.model(
    "Cwe",
    {
        "name": fields.String(
            description="Name of the CWE", example="Incorrect Selection of Fuse Values"
        ),
        "id": fields.String(description="ID of the CWE", example="1253"),
        "status": fields.String(description="Status of the CWE", example="Draft"),
        "weaknessabs": fields.String(description="Category of the CWE", example="Base"),
        "description": fields.String(
            description="Description of the CWE",
            example="Logic should be designed in a way that blown fuses do not put the product into an insecure state "
            "that can be leveraged by an attacker. Logic should be designed in a way that blown fuses do not "
            "put the product into an insecure state that can be leveraged by an attacker. ",
        ),
        "related_weaknesses": fields.List(
            fields.String,
            description="List of related weaknesses of the CWE",
            example=["693"],
        ),
    },
)


@api.route("/cwe")
@api.response(400, "Error processing request", model=message)
@api.response(404, "The requested CWE is not found", model=message)
@api.response(500, "Server error", model=message)
class CweId(Resource):
    @api.marshal_list_with(cwe, skip_none=True)
    def get(self):
        """
        List all CWE's

        Outputs a list of all CWEs (Common Weakness Enumeration).
        """
        from lib.DatabaseLayer import getCWEs

        cwes = getCWEs()

        if cwes is None:
            api.abort(404, "The requested CWE is not found")
        else:
            return cwes


@api.route("/cwe/<cwe_id>")
@api.param("cwe_id", "CWE id", example="1253")
@api.response(400, "Error processing request", model=message)
@api.response(404, "The requested CWE is not found", model=message)
@api.response(500, "Server error", model=message)
class CweId(Resource):
    @api.marshal_with(cwe, skip_none=True)
    def get(self, cwe_id):
        """
        CWE from CWE ID

        Outputs a specific CWE (Common Weakness Enumeration).
        """
        from lib.DatabaseLayer import getCWEs

        cwes = getCWEs(cwe_id)

        if cwes is None:
            api.abort(404, "The requested CWE is not found")
        else:
            return cwes



import logging
import random
import urllib
from datetime import timedelta

from CveXplore import CveXplore
from CveXplore.errors.database import DatabaseSchemaVersionError

from flask import Flask, render_template, request
from flask_bootstrap import Bootstrap
from flask_breadcrumbs import Breadcrumbs
from flask_login import LoginManager
from oauthlib.oauth2 import WebApplicationClient

from lib.Config import Configuration
from lib.LogHandler import AppLogger
from lib.Toolkit import isURL
from lib.User import User
from web.helpers.flask_authentication import FlaskAuthHandler
from web.helpers.flask_database import FlaskDatabaseHandler

login_manager = LoginManager()
auth_handler = FlaskAuthHandler()
dbh = FlaskDatabaseHandler()

config = Configuration()

cvex = None  # initialize CveXplore later

app = None
oidcClient = None

ACCESS_EXPIRES = timedelta(minutes=15)
REFRESH_EXPIRES = timedelta(days=30)

logging.setLoggerClass(AppLogger)


def create_app(version, run_path):
    global app, oidcClient, config

    app = Flask(__name__)

    app.config["version"] = version
    app.config["run_path"] = run_path

    if config.getWebInterface().lower() == "full":
        app.config["WebInterface"] = False
    else:
        app.config["WebInterface"] = True

    app.config["MONGO_DBNAME"] = config.getMongoDB()
    app.config["SECRET_KEY"] = str(random.getrandbits(256))
    app.config["RESTX_MASK_SWAGGER"] = False
    app.config["SWAGGER_UI_DOC_EXPANSION"] = "list"

    Breadcrumbs(app=app)
    Bootstrap(app)
    dbh.init_app(app)
    auth_handler.init_app(app)

    login_manager.init_app(app)
    login_manager.login_message = "You must be logged in to access this page!!!"
    login_manager.login_view = "auth.login"

    @app.before_first_request
    def init_cvexplore():
        # Defer CveXplore initialization to create MongoClient after fork
        # (avoids PyMongo 'opened before fork' warning)
        global cvex
        config.setMongoDBEnv()  # pass MongoDB settings to CveXplore
        cvex = CveXplore()

    # CORS
    @app.after_request
    def apply_caching(response):
        reqURL = request.base_url
        if (
            config.getCORS()
            and reqURL.count("/api/") == 1
            and reqURL.count("/admin") == 0
        ):
            response.headers.add(
                "Access-Control-Allow-Origin", config.getCORSAllowOrigin()
            )
        return response

    # OAuth 2 client setup
    if config.useOIDC():
        oidcClient = WebApplicationClient(config.getClientID())

    @login_manager.user_loader
    def load_user(id):
        return User.get(id, auth_handler)

    from .home import home as home_blueprint

    app.register_blueprint(home_blueprint)

    if not app.config["WebInterface"]:
        from .auth import auth as auth_blueprint

        app.register_blueprint(auth_blueprint)

        from .admin import admin as admin_blueprint

        app.register_blueprint(admin_blueprint, url_prefix="/admin")

    from .restapi import blueprint as api

    app.register_blueprint(api)

    @app.context_processor
    def version():
        def get_version():
            return app.config["version"]

        return dict(get_version=get_version)

    @app.context_processor
    def db_schema():
        def db_schema():
            try:
                return cvex.database.validate_schema()
            except DatabaseSchemaVersionError as err:
                return err

        return dict(db_schema=db_schema)

    @app.context_processor
    def WebInterface():
        def get_WebInterface():
            return app.config["WebInterface"]

        return dict(get_WebInterface=get_WebInterface)

    @app.context_processor
    def JSON2HTMLTable():
        # Doublequote, because we have to |safe the content for the tags
        def doublequote(data):
            return urllib.parse.quote_plus(urllib.parse.quote_plus(data))

        def JSON2HTMLTableFilter(data, stack=None):
            _return = ""
            if type(stack) == str:
                stack = [stack]

            if type(data) == list:
                if len(data) == 1:
                    _return += JSON2HTMLTableFilter(data[0], stack)
                else:
                    _return += '<ul class="via4">'
                    for item in data:
                        _return += "<li>%s</li>" % JSON2HTMLTableFilter(item, stack)
                    _return += "</ul>"
            elif type(data) == dict:
                _return += '<table class="invisiTable">'
                for key, val in sorted(data.items()):
                    _return += "<tr><td><b>%s</b></td><td>%s</td></tr>" % (
                        key,
                        JSON2HTMLTableFilter(val, stack + [key]),
                    )
                _return += "</table>"
            elif type(data) == str:
                if stack:
                    _return += (
                        "<a href='/link/"
                        + doublequote(".".join(stack))
                        + "/"
                        + doublequote(data)
                        + "'>"
                    )  # link opening
                    _return += "<i class='fas fa-link' aria-hidden='true'></i> </a>"
                _return += (
                    "<a target='_blank' href='%s'>%s</a>" % (data, data)
                    if isURL(data)
                    else data
                )
            _return += ""
            return _return

        return dict(JSON2HTMLTable=JSON2HTMLTableFilter)

    @app.template_filter("htmlEncode")
    def htmlEncode(string):
        return urllib.parse.quote_plus(string).lower()

    @app.template_filter("htmlDecode")
    def htmlDecode(string):
        return urllib.parse.unquote_plus(string)

    @app.template_filter("sortIntLikeStr")
    def sortIntLikeStr(datalist):
        return sorted(datalist, key=lambda k: int(k))

    @app.errorhandler(404)
    def page_not_found(error):
        return render_template("404.html"), 404

    return app



from flask import Blueprint
from flask_restx import Api

from web.run import app
from .browse_vendor import api as browse_api
from .capec import api as capec_api
from .cpe_convert import api as cpe_api
from .cve import api as cve_api
from .cwe import api as cwe_api
from .dbinfo import api as dbinfo_api
from .query import api as query_api
from .search_vendor import api as search_vendor_api

namespaces = [
    cpe_api,
    cve_api,
    cwe_api,
    capec_api,
    query_api,
    browse_api,
    search_vendor_api,
    dbinfo_api,
]

blueprint = Blueprint("api", __name__, url_prefix="/api")

api = Api(
    blueprint,
    title="CVE-Search",
    version=app.config["version"],
    description="CVE-Search API documentation",
)

for each in namespaces:
    api.add_namespace(each)
